@@ -14,11 +14,11 @@
  */
 
 #include <stdlib.h>
-#include <string>
 #include <stdio.h>
-#include <ctype.h>
 #include <time.h>
+#include <ctype.h>
 
+#include <string>
 #include <vector>
 #include <map>
 
@@ -40,18 +40,21 @@
 #include "XPM.h"
 #include "LineMarker.h"
 #include "Style.h"
-#include "AutoComplete.h"
 #include "ViewStyle.h"
 #include "CharClassify.h"
 #include "Decoration.h"
 #include "CaseFolder.h"
 #include "Document.h"
+#include "CaseConvert.h"
 #include "Selection.h"
 #include "PositionCache.h"
+#include "EditModel.h"
+#include "MarginView.h"
+#include "EditView.h"
 #include "Editor.h"
 
+#include "AutoComplete.h"
 #include "ScintillaBase.h"
-#include "CaseConvert.h"
 
 extern "C" NSString* ScintillaRecPboardType;
 
@@ -111,8 +114,8 @@ private:
   FindHighlightLayer *layerFindIndicator;
 
 protected:
-  Point GetVisibleOriginInMain();
-  PRectangle GetClientRectangle();
+  Point GetVisibleOriginInMain() const;
+  PRectangle GetClientRectangle() const;
   virtual PRectangle GetClientDrawingRectangle();
   Point ConvertPoint(NSPoint point);
   virtual void RedrawRect(PRectangle rc);
@@ -133,8 +136,7 @@ public:
   void RegisterNotifyCallback(intptr_t windowid, SciNotifyFunc callback);
   sptr_t WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam);
 
-  ScintillaView* TopContainer();
-  NSScrollView* ScrollContainer();
+  NSScrollView* ScrollContainer() const;
   SCIContentView* ContentView();
 
   bool SyncPaint(void* gc, PRectangle rc);
@@ -142,7 +144,11 @@ public:
   void PaintMargin(NSRect aRect);
 
   virtual sptr_t DefWndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam);
-  void SetTicking(bool on);
+  void TickFor(TickReason reason);
+  bool FineTickerAvailable();
+  bool FineTickerRunning(TickReason reason);
+  void FineTickerStart(TickReason reason, int millis, int tolerance);
+  void FineTickerCancel(TickReason reason);
   bool SetIdle(bool on);
   void SetMouseCapture(bool on);
   bool HaveMouseCapture();
@@ -179,6 +185,7 @@ public:
 
   static sptr_t DirectFunction(sptr_t ptr, unsigned int iMessage, uptr_t wParam, sptr_t lParam);
 
+  NSTimer *timers[tickPlatform+1];
   void TimerFired(NSTimer* timer);
   void IdleTimerFired();
   static void UpdateObserver(CFRunLoopObserverRef observer, CFRunLoopActivity activity, void *sci);
