@@ -11,25 +11,47 @@
 
 #import <Cocoa/Cocoa.h>
 
-#import "Platform.h"
 #import "Scintilla.h"
 #import "SciLexer.h"
 
 #import "InfoBarCommunicator.h"
-#import "ScintillaCocoa.h"
+
+/**
+ * Scintilla sends these two messages to the notify handler. Please refer
+ * to the Windows API doc for details about the message format.
+ */
+#define WM_COMMAND 1001
+#define WM_NOTIFY 1002
+
+namespace Scintilla {
+/**
+ * On the Mac, there is no WM_COMMAND or WM_NOTIFY message that can be sent
+ * back to the parent. Therefore, there must be a callback handler that acts
+ * like a Windows WndProc, where Scintilla can send notifications to. Use
+ * ScintillaView registerNotifyCallback() to register such a handler.
+ * Message format is:
+ * <br>
+ * WM_COMMAND: HIWORD (wParam) = notification code, LOWORD (wParam) = control ID, lParam = ScintillaCocoa*
+ * <br>
+ * WM_NOTIFY: wParam = control ID, lParam = ptr to SCNotification structure, with hwndFrom set to ScintillaCocoa*
+ */
+typedef void(*SciNotifyFunc) (intptr_t windowid, unsigned int iMessage, uintptr_t wParam, uintptr_t lParam);
+
+class ScintillaCocoa;
+}
 
 @class ScintillaView;
 
-extern NSString *SCIUpdateUINotification;
+extern NSString *const SCIUpdateUINotification;
 
 @protocol ScintillaNotificationProtocol
 - (void)notification: (Scintilla::SCNotification*)notification;
 @end
 
 /**
- * MarginView draws line numbers and other margins next to the text view.
+ * SCIMarginView draws line numbers and other margins next to the text view.
  */
-@interface MarginView : NSRulerView
+@interface SCIMarginView : NSRulerView
 {
 @private
   int marginWidth;
@@ -45,15 +67,15 @@ extern NSString *SCIUpdateUINotification;
 @end
 
 /**
- * InnerView is the Cocoa interface to the Scintilla backend. It handles text input and
+ * SCIContentView is the Cocoa interface to the Scintilla backend. It handles text input and
  * provides a canvas for painting the output.
  */
-@interface InnerView : NSView <NSTextInputClient, NSUserInterfaceValidations>
+@interface SCIContentView : NSView <NSTextInputClient, NSUserInterfaceValidations>
 {
 @private
   ScintillaView* mOwner;
   NSCursor* mCurrentCursor;
-  NSTrackingRectTag mCurrentTrackingRect;
+  NSTrackingArea *trackingArea;
 
   // Set when we are in composition mode and partial input is displayed.
   NSRange mMarkedTextRange;
@@ -62,34 +84,32 @@ extern NSString *SCIUpdateUINotification;
 
 @property (nonatomic, assign) ScintillaView* owner;
 
-- (void) dealloc;
 - (void) removeMarkedText;
-- (void) setCursor: (Scintilla::Window::Cursor) cursor;
+- (void) setCursor: (int) cursor;
 
 - (BOOL) canUndo;
 - (BOOL) canRedo;
 
 @end
 
-@interface ScintillaView : NSView <InfoBarCommunicator>
+@interface ScintillaView : NSView <InfoBarCommunicator, ScintillaNotificationProtocol>
 {
 @private
   // The back end is kind of a controller and model in one.
   // It uses the content view for display.
   Scintilla::ScintillaCocoa* mBackend;
-  
+
   // This is the actual content to which the backend renders itself.
-  InnerView* mContent;
-  
+  SCIContentView* mContent;
+
   NSScrollView *scrollView;
-  MarginView *marginView;
-  
+  SCIMarginView *marginView;
+
   CGFloat zoomDelta;
-  
+
   // Area to display additional controls (e.g. zoom info, caret position, status info).
   NSView <InfoBarCommunicator>* mInfoBar;
   BOOL mInfoBarAtTop;
-  int mInitialInfoBarWidth;
 
   id<ScintillaNotificationProtocol> mDelegate;
 }
@@ -98,31 +118,32 @@ extern NSString *SCIUpdateUINotification;
 @property (nonatomic, assign) id<ScintillaNotificationProtocol> delegate;
 @property (nonatomic, readonly) NSScrollView *scrollView;
 
-- (void) dealloc;
-- (void) positionSubViews;
++ (Class) contentViewClass;
 
-- (void) sendNotification: (NSString*) notificationName;
 - (void) notify: (NotificationType) type message: (NSString*) message location: (NSPoint) location
           value: (float) value;
 - (void) setCallback: (id <InfoBarCommunicator>) callback;
 
 - (void) suspendDrawing: (BOOL) suspend;
+- (void) notification: (Scintilla::SCNotification*) notification;
 
 // Scroller handling
 - (void) setMarginWidth: (int) width;
-- (void) scrollerAction: (id) sender;
-- (InnerView*) content;
+- (SCIContentView*) content;
+- (void) updateMarginCursors;
 
 // NSTextView compatibility layer.
 - (NSString*) string;
 - (void) setString: (NSString*) aString;
-- (void) insertText: (NSString*) aString;
+- (void) insertText: (id) aString;
 - (void) setEditable: (BOOL) editable;
 - (BOOL) isEditable;
 - (NSRange) selectedRange;
 
 - (NSString*) selectedString;
 
+- (void) deleteRange: (NSRange) range;
+
 - (void)setFontName: (NSString*) font
                size: (int) size
                bold: (BOOL) bold
@@ -153,7 +174,8 @@ extern NSString *SCIUpdateUINotification;
 - (void) setLexerProperty: (NSString*) name value: (NSString*) value;
 - (NSString*) getLexerProperty: (NSString*) name;
 
-- (void) registerNotifyCallback: (intptr_t) windowid value: (Scintilla::SciNotifyFunc) callback;
+// The delegate property should be used instead of registerNotifyCallback which is deprecated.
+- (void) registerNotifyCallback: (intptr_t) windowid value: (Scintilla::SciNotifyFunc) callback __attribute__((deprecated));
 
 - (void) setInfoBar: (NSView <InfoBarCommunicator>*) aView top: (BOOL) top;
 - (void) setStatusText: (NSString*) text;
