@@ -7,7 +7,7 @@
 
 class DirectorExtension : public Extension {
 private:
-	DirectorExtension() : inputWatcher(-1), inputChannel(0) {} // Singleton
+	DirectorExtension() : inputWatcher(-1), inputChannel(0), host(0) {} // Singleton
 	DirectorExtension(const DirectorExtension &); // Disable copy ctor
 	void operator=(const DirectorExtension &);    // Disable operator=
 
@@ -43,5 +43,6 @@ public:
 private:
 	int inputWatcher;
 	GIOChannel *inputChannel;
+	ExtensionAPI *host;
 };
 
