@@ -471,7 +471,7 @@ void SurfaceImpl::MeasureWidths(Font &font,
 		int i=0;
 		while (ui<fit) {
 			size_t lenChar = utf8LengthFromLead(us[i]);
-			size_t codeUnits = (lenChar < 4) ? 1 : 2;
+			int codeUnits = (lenChar < 4) ? 1 : 2;
 			qreal xPosition = tl.cursorToX(ui+codeUnits);
 			for (unsigned int bytePos=0; (bytePos<lenChar) && (i<len); bytePos++) {
 				positions[i++] = qRound(xPosition);
@@ -528,7 +528,7 @@ XYPOSITION SurfaceImpl::Descent(Font &font)
 	QFontMetrics metrics(*FontPointer(font), device);
 	// Qt returns 1 less than true descent
 	// See: QFontEngineWin::descent which says:
-	// ### we substract 1 to even out the historical +1 in QFontMetrics's
+	// ### we subtract 1 to even out the historical +1 in QFontMetrics's
 	// ### height=asc+desc+1 equation. Fix in Qt5.
 	return metrics.descent() + 1;
 }
@@ -738,8 +738,7 @@ PRectangle Window::GetMonitorRect(Point pt)
 	QPoint posGlobal = window(wid)->mapToGlobal(QPoint(pt.x, pt.y));
 	QDesktopWidget *desktop = QApplication::desktop();
 	QRect rectScreen = desktop->availableGeometry(posGlobal);
-	rectScreen.moveLeft(-originGlobal.x());
-	rectScreen.moveTop(-originGlobal.y());
+	rectScreen.translate(-originGlobal.x(), -originGlobal.y());
 	return PRectangle(rectScreen.left(), rectScreen.top(),
 	        rectScreen.right(), rectScreen.bottom());
 }
@@ -770,6 +769,7 @@ public:
 	virtual void RegisterImage(int type, const char *xpmData);
 	virtual void RegisterRGBAImage(int type, int width, int height,
 		const unsigned char *pixelsImage);
+	virtual void RegisterQPixmapImage(int type, const QPixmap& pm);
 	virtual void ClearRegisteredImages();
 	virtual void SetDoubleClickAction(CallBackAction action, void *data);
 	virtual void SetList(const char *list, char separator, char typesep);
@@ -832,6 +832,16 @@ void ListBoxImpl::Create(Window &parent,
 	list->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 	list->move(location.x, location.y);
 
+	int maxIconWidth = 0;
+	int maxIconHeight = 0;
+	foreach (QPixmap im, images) {
+		if (maxIconWidth < im.width())
+			maxIconWidth = im.width();
+		if (maxIconHeight < im.height())
+			maxIconHeight = im.height();
+	}
+	list->setIconSize(QSize(maxIconWidth, maxIconHeight));
+
 	wid = list;
 }
 
@@ -883,9 +893,16 @@ int ListBoxImpl::CaretFromEdge()
 			maxIconWidth = im.width();
 	}
 
-	// The '7' is from trial and error on Windows - there may be
+	int extra;
+	// The 12 is from trial and error on OS X and the 7
+	// is from trial and error on Windows - there may be
 	// a better programmatic way to find any padding factors.
-	return maxIconWidth  + (2 * list->frameWidth()) + 7;
+#ifdef Q_OS_DARWIN
+	extra = 12;
+#else
+	extra = 7;
+#endif
+	return maxIconWidth + (2 * list->frameWidth()) + extra;
 }
 
 void ListBoxImpl::Clear()
@@ -916,6 +933,13 @@ int ListBoxImpl::Length()
 void ListBoxImpl::Select(int n)
 {
 	ListWidget *list = static_cast<ListWidget *>(wid);
+	QModelIndex index = list->model()->index(n, 0);
+	if (index.isValid()) {
+		QRect row_rect = list->visualRect(index);
+		if (!list->viewport()->rect().contains(row_rect)) {
+			list->scrollTo(index, QAbstractItemView::PositionAtTop);
+		}
+	}
 	list->setCurrentRow(n);
 }
 
@@ -950,21 +974,39 @@ void ListBoxImpl::GetValue(int n, char *value, int len)
 	value[len-1] = '\0';
 }
 
+void ListBoxImpl::RegisterQPixmapImage(int type, const QPixmap& pm)
+{
+	images[type] = pm;
+
+	ListWidget *list = static_cast<ListWidget *>(wid);
+	if (list != NULL) {
+		QSize iconSize = list->iconSize();
+		if (pm.width() > iconSize.width() || pm.height() > iconSize.height())
+			list->setIconSize(QSize(qMax(pm.width(), iconSize.width()), 
+						 qMax(pm.height(), iconSize.height())));
+	}
+
+}
+
 void ListBoxImpl::RegisterImage(int type, const char *xpmData)
 {
-	images[type] = QPixmap(reinterpret_cast<const char * const *>(xpmData));
+	RegisterQPixmapImage(type, QPixmap(reinterpret_cast<const char * const *>(xpmData)));
 }
 
 void ListBoxImpl::RegisterRGBAImage(int type, int width, int height, const unsigned char *pixelsImage)
 {
 	std::vector<unsigned char> imageBytes = ImageByteSwapped(width, height, pixelsImage);
 	QImage image(&imageBytes[0], width, height, QImage::Format_ARGB32);
-	images[type] = QPixmap::fromImage(image);
+	RegisterQPixmapImage(type, QPixmap::fromImage(image));
 }
 
 void ListBoxImpl::ClearRegisteredImages()
 {
 	images.clear();
+	
+	ListWidget *list = static_cast<ListWidget *>(wid);
+	if (list != NULL)
+		list->setIconSize(QSize(0, 0));
 }
 
 void ListBoxImpl::SetDoubleClickAction(CallBackAction action, void *data)
@@ -978,7 +1020,7 @@ void ListBoxImpl::SetList(const char *list, char separator, char typesep)
 	// This method is *not* platform dependent.
 	// It is borrowed from the GTK implementation.
 	Clear();
-	int count = strlen(list) + 1;
+	size_t count = strlen(list) + 1;
 	std::vector<char> words(list, list+count);
 	char *startword = &words[0];
 	char *numword = NULL;
@@ -1082,7 +1124,7 @@ public:
 
 	virtual Function FindFunction(const char *name) {
 		if (lib) {
-			// C++ standard doesn't like casts betwen function pointers and void pointers so use a union
+			// C++ standard doesn't like casts between function pointers and void pointers so use a union
 			union {
 #if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 				QFunctionPointer fp;
