@@ -9,6 +9,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <stdio.h>
+#include <stddef.h>
 #include <fcntl.h>
 #include <sys/stat.h>
 #include <time.h>
@@ -19,28 +20,6 @@
 #include <map>
 #include <sstream>
 
-#if defined(__unix__)
-
-#include <unistd.h>
-
-#if defined(GTK)
-#include <gtk/gtk.h>
-#endif
-
-#else
-
-#undef _WIN32_WINNT
-#define _WIN32_WINNT  0x0500
-#include <windows.h>
-#include <commctrl.h>
-
-// For chdir
-#ifdef _MSC_VER
-#include <direct.h>
-#endif
-
-#endif
-
 #include "Scintilla.h"
 #include "ILexer.h"
 
@@ -317,7 +296,7 @@ void SciTEBase::SaveToRTF(FilePath saveName, int start, int end) {
 		fclose(fp);
 	} else {
 		GUI::gui_string msg = LocaliseMessage("Could not save file '^0'.", filePath.AsInternal());
-		WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
+		WindowMessageBox(wSciTE, msg);
 	}
 }
 
@@ -665,7 +644,7 @@ void SciTEBase::SaveToHTML(FilePath saveName) {
 	} else {
 		GUI::gui_string msg = LocaliseMessage(
 		            "Could not save file \"^0\".", filePath.AsInternal());
-		WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
+		WindowMessageBox(wSciTE, msg);
 	}
 }
 
@@ -1145,7 +1124,7 @@ void SciTEBase::SaveToPDF(FilePath saveName) {
 	if (!fp) {
 		// couldn't open the file for saving, issue an error message
 		GUI::gui_string msg = LocaliseMessage("Could not save file '^0'.", filePath.AsInternal());
-		WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
+		WindowMessageBox(wSciTE, msg);
 		return;
 	}
 	// initialise PDF rendering
@@ -1365,7 +1344,7 @@ void SciTEBase::SaveToTEX(FilePath saveName) {
 	} else {
 		GUI::gui_string msg = LocaliseMessage(
 		            "Could not save file \"^0\".", filePath.AsInternal());
-		WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
+		WindowMessageBox(wSciTE, msg);
 	}
 }
 
@@ -1544,6 +1523,6 @@ void SciTEBase::SaveToXML(FilePath saveName) {
 		fclose(fp);
 	} else {
 		GUI::gui_string msg = LocaliseMessage("Could not save file \"^0\".", filePath.AsInternal());
-		WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
+		WindowMessageBox(wSciTE, msg);
 	}
 }
