@@ -11,15 +11,12 @@ extern const GUI::gui_char propUserFileName[];
 extern const GUI::gui_char propGlobalFileName[];
 extern const GUI::gui_char propAbbrevFileName[];
 
-#ifdef WIN32
+#ifdef _WIN32
 #ifdef _MSC_VER
 // Shut up level 4 warning:
 // warning C4800: forcing value to bool 'true' or 'false' (performance warning)
 #pragma warning(disable: 4800)
 #endif
-#ifdef __DMC__
-#include <time.h>
-#endif
 #endif
 
 inline int Minimum(int a, int b) {
@@ -47,7 +44,7 @@ enum {
 struct SelectedRange {
 	int position;
 	int anchor;
-	SelectedRange(int position_= INVALID_POSITION, int anchor_= INVALID_POSITION) : 
+	SelectedRange(int position_= INVALID_POSITION, int anchor_= INVALID_POSITION) :
 		position(position_), anchor(anchor_) {
 	}
 };
@@ -380,7 +377,7 @@ protected:
 	int lexLPeg;
 	StringList apis;
 	SString apisFileNames;
-	SString functionDefinition;
+	std::string functionDefinition;
 
 	int diagnosticStyleStart;
 	enum { diagnosticStyles=4};
@@ -447,7 +444,6 @@ protected:
 	bool firstPropertiesRead;
 	bool splitVertical;	///< @c true if the split bar between editor and output is vertical.
 	bool bufferedDraw;
-	bool twoPhaseDraw;
 	bool bracesCheck;
 	bool bracesSloppy;
 	int bracesStyle;
@@ -592,7 +588,7 @@ protected:
 	std::string DiscoverLanguage();
 	void OpenFile(long fileSize, bool suppressMessage, bool asynchronous);
 	virtual void OpenUriList(const char *) {}
-	virtual bool OpenDialog(FilePath directory, const GUI::gui_char *filter) = 0;
+	virtual bool OpenDialog(FilePath directory, const GUI::gui_char *filesFilter) = 0;
 	virtual bool SaveAsDialog() = 0;
 	virtual void LoadSessionDialog() {}
 	virtual void SaveSessionDialog() {}
@@ -621,9 +617,13 @@ protected:
 	    sfProgressVisible = 1, 	// Show in background save strip
 	    sfSynchronous = 16	// Write synchronously blocking UI
 	};
-	int SaveIfUnsure(bool forceQuestion = false, SaveFlags sf = sfProgressVisible);
-	int SaveIfUnsureAll(bool forceQuestion = false);
-	int SaveIfUnsureForBuilt();
+	enum SaveResult {
+		saveCompleted,
+		saveCancelled
+	};
+	SaveResult SaveIfUnsure(bool forceQuestion = false, SaveFlags sf = sfProgressVisible);
+	SaveResult SaveIfUnsureAll();
+	SaveResult SaveIfUnsureForBuilt();
 	bool SaveIfNotOpen(const FilePath &destFile, bool fixCase);
 	void AbandonAutomaticSave();
 	bool Save(SaveFlags sf = sfProgressVisible);
@@ -668,8 +668,10 @@ protected:
 	static SString GetRange(GUI::ScintillaWindow &win, int selStart, int selEnd);
 	virtual SString GetRangeInUIEncoding(GUI::ScintillaWindow &win, int selStart, int selEnd);
 	static SString GetLine(GUI::ScintillaWindow &win, int line);
+	void RangeExtend(GUI::ScintillaWindow &wCurrent, int &selStart, int &selEnd,
+		bool (SciTEBase::*ischarforsel)(char ch));
 	SString RangeExtendAndGrab(GUI::ScintillaWindow &wCurrent, int &selStart, int &selEnd,
-	        bool (SciTEBase::*ischarforsel)(char ch), bool stripEol = true);
+		bool (SciTEBase::*ischarforsel)(char ch), bool stripEol = true);
 	SString SelectionExtend(bool (SciTEBase::*ischarforsel)(char ch), bool stripEol = true);
 	SString SelectionWord(bool stripEol = true);
 	SString SelectionFilename();
@@ -677,9 +679,24 @@ protected:
 	void SelectionIntoFind(bool stripEol = true);
 	virtual std::string EncodeString(const std::string &s);
 	virtual void Find() = 0;
-	virtual int WindowMessageBox(GUI::Window &w, const GUI::gui_string &msg, int style) = 0;
+	enum MessageBoxChoice {
+		mbOK,
+		mbCancel,
+		mbYes,
+		mbNo
+	};
+	typedef int MessageBoxStyle;
+	enum {
+		// Same as Win32 MB_*
+		mbsOK = 0,
+		mbsYesNo = 4,
+		mbsYesNoCancel = 3,
+		mbsIconQuestion = 0x20,
+		mbsIconWarning = 0x30
+	};
+	virtual MessageBoxChoice WindowMessageBox(GUI::Window &w, const GUI::gui_string &msg, MessageBoxStyle style = mbsIconWarning) = 0;
 	virtual void FindMessageBox(const SString &msg, const std::string *findItem = 0) = 0;
-	int FindInTarget(std::string findWhat, int startPosition, int endPosition);
+	int FindInTarget(std::string findWhatText, int startPosition, int endPosition);
 	virtual void SetFindText(const char *sFind);
 	virtual void SetFind(const char *sFind);
 	virtual bool FindHasText() const;
@@ -757,7 +774,7 @@ protected:
 	virtual void QuitProgram() = 0;
 	void CloseTab(int tab);
 	void CloseAllBuffers(bool loadingSession = false);
-	int SaveAllBuffers(bool forceQuestion, bool alwaysYes = false);
+	SaveResult SaveAllBuffers(bool alwaysYes);
 	void SaveTitledBuffers();
 	virtual void CopyAsRTF() {}
 	virtual void CopyPath() {}
@@ -846,7 +863,7 @@ protected:
 	virtual void ReadProperties();
 	void SetOneStyle(GUI::ScintillaWindow &win, int style, const StyleDefinition &sd);
 	void SetStyleBlock(GUI::ScintillaWindow &win, const char *lang, int start, int last);
-	void SetStyleFor(GUI::ScintillaWindow &win, const char *language);
+	void SetStyleFor(GUI::ScintillaWindow &win, const char *lang);
 	static void SetOneIndicator(GUI::ScintillaWindow &win, int indicator, const IndicatorDefinition &ind);
 	void ReloadProperties();
 
@@ -934,19 +951,6 @@ private:
 	void operator=(const SciTEBase&);
 };
 
-#if defined(__unix__)
-// MessageBox
-#define MB_OK	(0L)
-#define MB_YESNO	(0x4L)
-#define MB_YESNOCANCEL	(0x3L)
-#define MB_ICONWARNING	(0x30L)
-#define MB_ICONQUESTION (0x20L)
-#define IDOK	(1)
-#define IDCANCEL	(2)
-#define IDYES	(6)
-#define IDNO	(7)
-#endif
-
 int ControlIDOfCommand(unsigned long);
 long ColourOfProperty(PropSetFile &props, const char *key, Colour colourDefault);
 void WindowSetFocus(GUI::ScintillaWindow &w);
