@@ -9,6 +9,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <stdio.h>
+#include <stddef.h>
 #include <time.h>
 #include <assert.h>
 
@@ -17,31 +18,6 @@
 #include <set>
 #include <map>
 
-#if defined(__unix__)
-
-#include <unistd.h>
-
-#if defined(GTK)
-#include <gtk/gtk.h>
-#endif
-
-#else
-
-#undef _WIN32_WINNT
-#define _WIN32_WINNT  0x0500
-#include <windows.h>
-#include <commctrl.h>
-
-// For chdir
-#ifdef _MSC_VER
-#include <direct.h>
-#endif
-#ifdef __DMC__
-#include <dir.h>
-#endif
-
-#endif
-
 #include "Scintilla.h"
 #include "SciLexer.h"
 #include "ILexer.h"
@@ -503,7 +479,7 @@ bool SciTEBase::CanMakeRoom(bool maySaveIfDirty) {
 		return true;
 	} else if (maySaveIfDirty) {
 		// All available buffers are taken, try and close the current one
-		if (SaveIfUnsure(true, static_cast<SaveFlags>(sfProgressVisible | sfSynchronous)) != IDCANCEL) {
+		if (SaveIfUnsure(true, static_cast<SaveFlags>(sfProgressVisible | sfSynchronous)) != saveCancelled) {
 			// The file isn't dirty, or the user agreed to close the current one
 			return true;
 		}
@@ -735,7 +711,7 @@ void SciTEBase::SaveSessionFile(const GUI::gui_char *sessionName) {
 		if (!mem.empty()) {
 			fprintf(sessionFile, "\n");
 			it = mem.begin();
-			for (int i = 0; it != mem.end(); i++, it++) {
+			for (int i = 0; it != mem.end(); i++, ++it) {
 				propKey = IndexPropKey("search", i, "findwhat");
 				fprintf(sessionFile, "%s=%s\n", propKey.c_str(), (*it).c_str());
 			}
@@ -746,7 +722,7 @@ void SciTEBase::SaveSessionFile(const GUI::gui_char *sessionName) {
 			fprintf(sessionFile, "\n");
 			mem = memReplaces.AsVector();
 			it = mem.begin();
-			for (int i = 0; it != mem.end(); i++, it++) {
+			for (int i = 0; it != mem.end(); i++, ++it) {
 				propKey = IndexPropKey("search", i, "replacewith");
 				fprintf(sessionFile, "%s=%s\n", propKey.c_str(), (*it).c_str());
 			}
@@ -1013,14 +989,14 @@ void SciTEBase::Close(bool updateUI, bool loadingSession, bool makingRoomForNew)
 void SciTEBase::CloseTab(int tab) {
 	int tabCurrent = buffers.Current();
 	if (tab == tabCurrent) {
-		if (SaveIfUnsure() != IDCANCEL) {
+		if (SaveIfUnsure() != saveCancelled) {
 			Close();
 			WindowSetFocus(wEditor);
 		}
 	} else {
 		FilePath fpCurrent = buffers.buffers[tabCurrent].AbsolutePath();
 		SetDocumentAt(tab);
-		if (SaveIfUnsure() != IDCANCEL) {
+		if (SaveIfUnsure() != saveCancelled) {
 			Close();
 			WindowSetFocus(wEditor);
 			// Return to the previous buffer
@@ -1030,7 +1006,7 @@ void SciTEBase::CloseTab(int tab) {
 }
 
 void SciTEBase::CloseAllBuffers(bool loadingSession) {
-	if (SaveAllBuffers(false) != IDCANCEL) {
+	if (SaveAllBuffers(false) != saveCancelled) {
 		while (buffers.lengthVisible > 1)
 			Close(false, loadingSession);
 
@@ -1038,19 +1014,19 @@ void SciTEBase::CloseAllBuffers(bool loadingSession) {
 	}
 }
 
-int SciTEBase::SaveAllBuffers(bool forceQuestion, bool alwaysYes) {
-	int choice = IDYES;
+SciTEBase::SaveResult SciTEBase::SaveAllBuffers(bool alwaysYes) {
+	SaveResult choice = saveCompleted;
 	UpdateBuffersCurrent();
 	int currentBuffer = buffers.Current();
-	for (int i = 0; (i < buffers.lengthVisible) && (choice != IDCANCEL); i++) {
+	for (int i = 0; (i < buffers.lengthVisible) && (choice != saveCancelled); i++) {
 		if (buffers.buffers[i].isDirty) {
 			SetDocumentAt(i);
 			if (alwaysYes) {
 				if (!Save()) {
-					choice = IDCANCEL;
+					choice = saveCancelled;
 				}
 			} else {
-				choice = SaveIfUnsure(forceQuestion);
+				choice = SaveIfUnsure(false);
 			}
 		}
 	}
@@ -1157,12 +1133,12 @@ void SciTEBase::SetBuffersMenu() {
 			GUI::gui_string entry;
 			GUI::gui_string titleTab;
 
-#if defined(WIN32) || defined(GTK)
+#if defined(_WIN32) || defined(GTK)
 			if (pos < 10) {
 				GUI::gui_string sPos = GUI::StringFromInteger((pos + 1) % 10);
 				GUI::gui_string sHotKey = GUI_TEXT("&") + sPos + GUI_TEXT(" ");
 				entry = sHotKey;	// hotkey 1..0
-#if defined(WIN32)
+#if defined(_WIN32)
 				titleTab = sHotKey; // add hotkey to the tabbar
 #elif defined(GTK)
 				titleTab = sPos + GUI_TEXT(" ");
@@ -1179,7 +1155,7 @@ void SciTEBase::SetBuffersMenu() {
 				GUI::gui_string filename = buffers.buffers[pos].Name().AsInternal();
 
 				EscapeFilePathsForMenu(path);
-#if defined(WIN32)
+#if defined(_WIN32)
 				// On Windows, '&' are also interpreted in tab names, so we need
 				// the escaped filename
 				EscapeFilePathsForMenu(filename);
@@ -1231,7 +1207,7 @@ void SciTEBase::SetFileStackMenu() {
 			if (recentFileStack[stackPos].IsSet()) {
 				GUI::gui_string sEntry;
 
-#if defined(WIN32) || defined(GTK)
+#if defined(_WIN32) || defined(GTK)
 				GUI::gui_string sPos = GUI::StringFromInteger((stackPos + 1) % 10);
 				GUI::gui_string sHotKey = GUI_TEXT("&") + sPos + GUI_TEXT(" ");
 				sEntry = sHotKey;
@@ -1491,7 +1467,7 @@ void SciTEBase::ToolsMenu(int item) {
 		if (jobQueue.IsExecuting() && (jobMode.jobType != jobImmediate))
 			// Busy running a tool and running a second can cause failures.
 			return;
-		if (jobMode.saveBefore == 2 || (jobMode.saveBefore == 1 && (!(CurrentBuffer()->isDirty) || Save())) || SaveIfUnsure() != IDCANCEL) {
+		if (jobMode.saveBefore == 2 || (jobMode.saveBefore == 1 && (!(CurrentBuffer()->isDirty) || Save())) || SaveIfUnsure() != saveCancelled) {
 			if (jobMode.isFilter)
 				CurrentBuffer()->fileModTime -= 1;
 			if (jobMode.jobType == jobImmediate) {
