@@ -9,6 +9,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <stdio.h>
+#include <stddef.h>
 #include <fcntl.h>
 #include <time.h>
 
@@ -17,28 +18,6 @@
 #include <set>
 #include <map>
 
-#if defined(__unix__)
-
-#include <unistd.h>
-
-#if defined(GTK)
-#include <gtk/gtk.h>
-#endif
-
-#else
-
-#undef _WIN32_WINNT
-#define _WIN32_WINNT  0x0500
-#include <windows.h>
-#include <commctrl.h>
-
-// For chdir
-#ifdef _MSC_VER
-#include <direct.h>
-#endif
-
-#endif
-
 #include "Scintilla.h"
 #include "ILexer.h"
 
@@ -264,7 +243,7 @@ void SciTEBase::OpenFile(long fileSize, bool suppressMessage, bool asynchronous)
 		// Already performing an asynchronous load or save so do not restart load
 		if (!suppressMessage) {
 			GUI::gui_string msg = LocaliseMessage("Could not open file '^0'.", filePath.AsInternal());
-			WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
+			WindowMessageBox(wSciTE, msg);
 		}
 		return;
 	}
@@ -273,7 +252,7 @@ void SciTEBase::OpenFile(long fileSize, bool suppressMessage, bool asynchronous)
 	if (!fp) {
 		if (!suppressMessage) {
 			GUI::gui_string msg = LocaliseMessage("Could not open file '^0'.", filePath.AsInternal());
-			WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
+			WindowMessageBox(wSciTE, msg);
 		}
 		if (!wEditor.Call(SCI_GETUNDOCOLLECTION)) {
 			wEditor.Call(SCI_SETUNDOCOLLECTION, 1);
@@ -332,7 +311,7 @@ void SciTEBase::TextRead(FileWorker *pFileWorker) {
 		buffers.buffers[iBuffer].lifeState = Buffer::readAll;
 		if (pFileLoader->err) {
 			GUI::gui_string msg = LocaliseMessage("Could not open file '^0'.", pFileLoader->path.AsInternal());
-			WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
+			WindowMessageBox(wSciTE, msg);
 			// Should refuse to save when failure occurs
 			buffers.buffers[iBuffer].lifeState = Buffer::empty;
 		}
@@ -424,7 +403,7 @@ void SciTEBase::TextWritten(FileWorker *pFileWorker) {
 		// Complete and release
 		buffers.buffers[iBuffer].CompleteStoring();
 		if (errSaved || cancelledSaved) {
-			// Background save failed (possibly out-of-space) so resurrect the 
+			// Background save failed (possibly out-of-space) so resurrect the
 			// buffer so can be saved to another disk or retried after making room.
 			buffers.SetVisible(iBuffer, true);
 			SetBuffersMenu();
@@ -452,12 +431,12 @@ void SciTEBase::TextWritten(FileWorker *pFileWorker) {
 		}
 	} else {
 		GUI::gui_string msg = LocaliseMessage("Could not find buffer '^0'.", pathSaved.AsInternal());
-		WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
+		WindowMessageBox(wSciTE, msg);
 	}
 
 	if (errSaved) {
 		GUI::gui_string msg = LocaliseMessage("Could not save file '^0'.", pathSaved.AsInternal());
-		WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
+		WindowMessageBox(wSciTE, msg);
 	}
 
 	if (IsPropertiesFile(pathSaved)) {
@@ -506,7 +485,7 @@ bool SciTEBase::Open(FilePath file, OpenFlags of) {
 	if (!absPath.IsUntitled() && absPath.IsDirectory()) {
 		GUI::gui_string msg = LocaliseMessage("Path '^0' is a directory so can not be opened.",
 			absPath.AsInternal());
-		WindowMessageBox(wSciTE, msg, MB_ICONWARNING);
+		WindowMessageBox(wSciTE, msg);
 		return false;
 	}
 
@@ -536,8 +515,8 @@ bool SciTEBase::Open(FilePath file, OpenFlags of) {
 			        "larger than the ^2 bytes limit set in the properties.\n"
 			        "Do you still want to open it?",
 			        absPath.AsInternal(), sSize.c_str(), sMaxSize.c_str());
-			int answer = WindowMessageBox(wSciTE, msg, MB_YESNO | MB_ICONWARNING);
-			if (answer != IDYES) {
+			MessageBoxChoice answer = WindowMessageBox(wSciTE, msg, mbsYesNo | mbsIconWarning);
+			if (answer != mbYes) {
 				return false;
 			}
 		}
@@ -586,7 +565,7 @@ bool SciTEBase::Open(FilePath file, OpenFlags of) {
 			wEditor.Call(SCI_SETUNDOCOLLECTION, 0);
 		}
 
-		asynchronous = (size > props.GetInt("background.open.size", -1)) && 
+		asynchronous = (size > props.GetInt("background.open.size", -1)) &&
 			!(of & (ofPreserveUndo|ofSynchronous));
 		OpenFile(size, of & ofQuiet, asynchronous);
 
@@ -755,8 +734,8 @@ void SciTEBase::CheckReload() {
 						          "The file '^0' has been modified outside SciTE. Should it be reloaded?",
 						          FileNameExt().AsInternal());
 					}
-					int decision = WindowMessageBox(wSciTE, msg, MB_YESNO);
-					if (decision == IDYES) {
+					MessageBoxChoice decision = WindowMessageBox(wSciTE, msg, mbsYesNo | mbsIconQuestion);
+					if (decision == mbYes) {
 						Open(filePath, static_cast<OpenFlags>(of | ofForceLoad));
 						DisplayAround(rf);
 					}
@@ -801,13 +780,14 @@ FilePath SciTEBase::SaveName(const char *ext) const {
 	return FilePath(savePath.c_str());
 }
 
-int SciTEBase::SaveIfUnsure(bool forceQuestion, SaveFlags sf) {
+SciTEBase::SaveResult SciTEBase::SaveIfUnsure(bool forceQuestion, SaveFlags sf) {
+	CurrentBuffer()->failedSave = false;
 	if (CurrentBuffer()->pFileWorker) {
 		if (CurrentBuffer()->pFileWorker->IsLoading())
 			// In semi-loaded state so refuse to save
-			return IDCANCEL;
+			return saveCancelled;
 		else
-			return IDNO;
+			return saveCompleted;
 	}
 	if ((CurrentBuffer()->isDirty) && (LengthDocument() || !filePath.IsUntitled() || forceQuestion)) {
 		if (props.GetInt("are.you.sure", 1) ||
@@ -819,23 +799,23 @@ int SciTEBase::SaveIfUnsure(bool forceQuestion, SaveFlags sf) {
 			} else {
 				msg = LocaliseMessage("Save changes to (Untitled)?");
 			}
-			int decision = WindowMessageBox(wSciTE, msg, MB_YESNOCANCEL | MB_ICONQUESTION);
-			if (decision == IDYES) {
+			MessageBoxChoice decision = WindowMessageBox(wSciTE, msg, mbsYesNoCancel | mbsIconQuestion);
+			if (decision == mbYes) {
 				if (!Save(sf))
-					decision = IDCANCEL;
+					return saveCancelled;
 			}
-			return decision;
+			return (decision == mbCancel) ? saveCancelled : saveCompleted;
 		} else {
 			if (!Save(sf))
-				return IDCANCEL;
+				return saveCancelled;
 		}
 	}
-	return IDYES;
+	return saveCompleted;
 }
 
-int SciTEBase::SaveIfUnsureAll(bool forceQuestion) {
-	if (SaveAllBuffers(forceQuestion) == IDCANCEL) {
-		return IDCANCEL;
+SciTEBase::SaveResult SciTEBase::SaveIfUnsureAll() {
+	if (SaveAllBuffers(false) == saveCancelled) {
+		return saveCancelled;
 	}
 	if (props.GetInt("save.recent")) {
 		for (int i = 0; i < buffers.lengthVisible; ++i) {
@@ -868,12 +848,12 @@ int SciTEBase::SaveIfUnsureAll(bool forceQuestion) {
 		}
 	}
 	// Initial document will be deleted when editor deleted
-	return IDYES;
+	return saveCompleted;
 }
 
-int SciTEBase::SaveIfUnsureForBuilt() {
+SciTEBase::SaveResult SciTEBase::SaveIfUnsureForBuilt() {
 	if (props.GetInt("save.all.for.build")) {
-		return SaveAllBuffers(false, !props.GetInt("are.you.sure.for.build"));
+		return SaveAllBuffers(!props.GetInt("are.you.sure.for.build"));
 	}
 	if (CurrentBuffer()->isDirty) {
 		if (props.GetInt("are.you.sure.for.build"))
@@ -881,7 +861,7 @@ int SciTEBase::SaveIfUnsureForBuilt() {
 
 		Save();
 	}
-	return IDYES;
+	return saveCompleted;
 }
 
 void SciTEBase::StripTrailingSpaces() {
@@ -968,7 +948,7 @@ bool SciTEBase::SaveBuffer(FilePath saveName, SaveFlags sf) {
 					retVal = true;
 				} else {
 					GUI::gui_string msg = LocaliseMessage("Failed to save file '^0' as thread could not be started.", saveName.AsInternal());
-					WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
+					WindowMessageBox(wSciTE, msg);
 				}
 			} else {
 				Utf8_16_Write convert;
@@ -1024,7 +1004,7 @@ bool SciTEBase::Save(SaveFlags sf) {
 			msg = LocaliseMessage(
 				"The file '^0' has not yet been loaded entirely, so it can not be saved right now. Please retry in a while.",
 				filePath.AsInternal());
-			WindowMessageBox(wSciTE, msg, MB_ICONWARNING);
+			WindowMessageBox(wSciTE, msg);
 			// It is OK to not save this file
 			return true;
 		}
@@ -1033,13 +1013,11 @@ bool SciTEBase::Save(SaveFlags sf) {
 			msg = LocaliseMessage(
 				"The file '^0' is already being saved.",
 				filePath.AsInternal());
-			WindowMessageBox(wSciTE, msg, MB_ICONWARNING);
+			WindowMessageBox(wSciTE, msg);
 			// It is OK to not save this file
 			return true;
 		}
 
-		int decision;
-
 		if (props.GetInt("save.deletes.first")) {
 			filePath.Remove();
 		} else if (props.GetInt("save.check.modified.time")) {
@@ -1048,8 +1026,8 @@ bool SciTEBase::Save(SaveFlags sf) {
 				(newModTime != CurrentBuffer()->fileModTime)) {
 				msg = LocaliseMessage("The file '^0' has been modified outside SciTE. Should it be saved?",
 					filePath.AsInternal());
-				decision = WindowMessageBox(wSciTE, msg, MB_YESNO | MB_ICONWARNING);
-				if (decision == IDNO) {
+				MessageBoxChoice decision = WindowMessageBox(wSciTE, msg, mbsYesNo | mbsIconQuestion);
+				if (decision == mbNo) {
 					return false;
 				}
 			}
@@ -1071,8 +1049,8 @@ bool SciTEBase::Save(SaveFlags sf) {
 				CurrentBuffer()->failedSave = true;
 				msg = LocaliseMessage(
 					"Could not save file '^0'. Save under a different name?", filePath.AsInternal());
-				decision = WindowMessageBox(wSciTE, msg, MB_YESNO | MB_ICONWARNING);
-				if (decision == IDYES) {
+				MessageBoxChoice decision = WindowMessageBox(wSciTE, msg, mbsYesNo | mbsIconWarning);
+				if (decision == mbYes) {
 					return SaveAsDialog();
 				}
 			}
@@ -1103,7 +1081,7 @@ bool SciTEBase::SaveIfNotOpen(const FilePath &destFile, bool fixCase) {
 	if (index >= 0) {
 		GUI::gui_string msg = LocaliseMessage(
 			    "File '^0' is already open in another buffer.", destFile.AsInternal());
-		WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
+		WindowMessageBox(wSciTE, msg);
 		return false;
 	} else {
 		SaveAs(absPath.AsInternal(), fixCase);
