@@ -9,6 +9,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <stdio.h>
+#include <stddef.h>
 #include <fcntl.h>
 #include <time.h>
 #include <locale.h>
@@ -27,21 +28,10 @@
 
 #if defined(__unix__)
 
-#include <unistd.h>
-
-#if defined(GTK)
-#include <gtk/gtk.h>
-#endif
-
 const GUI::gui_char menuAccessIndicator[] = GUI_TEXT("_");
 
 #else
 
-#undef _WIN32_WINNT
-#define _WIN32_WINNT  0x0500
-#include <windows.h>
-#include <commctrl.h>
-
 const GUI::gui_char menuAccessIndicator[] = GUI_TEXT("&");
 
 #endif
@@ -506,7 +496,6 @@ static const char *propertiesToForward[] = {
 	"nsis.ignorecase",
 	"nsis.uservars",
 	"ps.level",
-	"ps.tokenize",
 	"sql.backslash.escapes",
 	"styling.within.preprocessor",
 	"tab.timmy.whinge.level",
@@ -661,7 +650,7 @@ void SciTEBase::ReadProperties() {
 	wEditor.CallString(SCI_SETKEYWORDS, 0, kw0.c_str());
 
 	for (int wl = 1; wl <= KEYWORDSET_MAX; wl++) {
-		SString kwk(wl+1);
+		std::string kwk = StdStringFromInteger(wl+1);
 		kwk += '.';
 		kwk.insert(0, "keywords");
 		SString kw = props.GetNewExpand(kwk.c_str(), fileNameForExtension.c_str());
@@ -930,9 +919,11 @@ void SciTEBase::ReadProperties() {
 	wEditor.Call(SCI_SETBUFFEREDDRAW, bufferedDraw);
 	wOutput.Call(SCI_SETBUFFEREDDRAW, bufferedDraw);
 
-	twoPhaseDraw = props.GetInt("two.phase.draw", 1);
-	wEditor.Call(SCI_SETTWOPHASEDRAW, twoPhaseDraw);
-	wOutput.Call(SCI_SETTWOPHASEDRAW, twoPhaseDraw);
+	int phasesDraw = props.GetInt("phases.draw", -1);
+	if (phasesDraw < 0 || phasesDraw > SC_PHASES_MULTIPLE)
+		phasesDraw = props.GetInt("two.phase.draw", 1) ? SC_PHASES_TWO : SC_PHASES_ONE;
+	wEditor.Call(SCI_SETPHASESDRAW, phasesDraw);
+	wOutput.Call(SCI_SETPHASESDRAW, phasesDraw);
 
 	wEditor.Call(SCI_SETLAYOUTCACHE, props.GetInt("cache.layout", SC_CACHE_CARET));
 	wOutput.Call(SCI_SETLAYOUTCACHE, props.GetInt("output.cache.layout", SC_CACHE_CARET));
@@ -1473,15 +1464,6 @@ void SciTEBase::ReadPropertiesInitial() {
 	}
 	// end load the user defined short cut props
 
-
-#if defined(WIN32)
-
-	if (tabMultiLine) {	// Windows specific!
-		long wl = ::GetWindowLong(reinterpret_cast<HWND>(wTabBar.GetID()), GWL_STYLE);
-		::SetWindowLong(reinterpret_cast<HWND>(wTabBar.GetID()), GWL_STYLE, wl | TCS_MULTILINE);
-	}
-#endif
-
 	FilePath homepath = GetSciteDefaultHome();
 	props.Set("SciteDefaultHome", homepath.AsUTF8().c_str());
 	homepath = GetSciteUserHome();
