@@ -10,6 +10,8 @@
 
 #include <string>
 #include <vector>
+#include <algorithm>
+#include <functional>
 
 #include "Scintilla.h"
 
@@ -26,12 +28,25 @@ bool EndsWith(GUI::gui_string const &s, GUI::gui_string const &end) {
 		(std::equal(s.begin() + s.size() - end.size(), s.end(), end.begin()));
 }
 
-int Substitute(GUI::gui_string &s, const GUI::gui_string &sFind, const GUI::gui_string &sReplace) {
+int Substitute(std::wstring &s, const std::wstring &sFind, const std::wstring &sReplace) {
 	int c = 0;
 	size_t lenFind = sFind.size();
 	size_t lenReplace = sReplace.size();
 	size_t posFound = s.find(sFind);
-	while (posFound != GUI::gui_string::npos) {
+	while (posFound != std::wstring::npos) {
+		s.replace(posFound, lenFind, sReplace);
+		posFound = s.find(sFind, posFound + lenReplace);
+		c++;
+	}
+	return c;
+}
+
+int Substitute(std::string &s, const std::string &sFind, const std::string &sReplace) {
+	int c = 0;
+	size_t lenFind = sFind.size();
+	size_t lenReplace = sReplace.size();
+	size_t posFound = s.find(sFind);
+	while (posFound != std::string::npos) {
 		s.replace(posFound, lenFind, sReplace);
 		posFound = s.find(sFind, posFound + lenReplace);
 		c++;
@@ -45,6 +60,18 @@ std::string StdStringFromInteger(int i) {
 	return std::string(number);
 }
 
+static int LowerCaseAZ(int c) {
+	if (c >= 'A' && c <= 'Z') {
+		return c - 'A' + 'a';
+	} else {
+		return c;
+	}
+}
+
+void LowerCaseAZ(std::string &s) {
+	std::transform(s.begin(), s.end(), s.begin(), std::ptr_fun<int, int>(LowerCaseAZ));
+}
+
 /**
  * Convert a string into C string literal form using \a, \b, \f, \n, \r, \t, \v, and \ooo.
  * The return value is a newly allocated character array containing the result.
@@ -190,11 +217,17 @@ unsigned int UnSlash(char *s) {
 	return static_cast<unsigned int>(o - sStart);
 }
 
+std::string UnSlashString(const char *s) {
+	std::string sCopy(s, strlen(s) + 1);
+	unsigned int len = UnSlash(&sCopy[0]);
+	return sCopy.substr(0, len);
+}
+
 /**
  * Convert C style \0oo into their indicated characters.
  * This is used to get control characters into the regular expresion engine.
  */
-unsigned int UnSlashLowOctal(char *s) {
+static unsigned int UnSlashLowOctal(char *s) {
 	char *sStart = s;
 	char *o = s;
 	while (*s) {
@@ -211,3 +244,9 @@ unsigned int UnSlashLowOctal(char *s) {
 	*o = '\0';
 	return static_cast<unsigned int>(o - sStart);
 }
+
+std::string UnSlashLowOctalString(const char *s) {
+	std::string sCopy(s, strlen(s) + 1);
+	unsigned int len = UnSlashLowOctal(&sCopy[0]);
+	return sCopy.substr(0, len);
+}
