@@ -7,10 +7,17 @@
 
 bool StartsWith(GUI::gui_string const &s, GUI::gui_string const &end);
 bool EndsWith(GUI::gui_string const &s, GUI::gui_string const &end);
-int Substitute(GUI::gui_string &s, const GUI::gui_string &sFind, const GUI::gui_string &sReplace);
+
+// Substitute is duplicated instead of templated as it was ambiguous when implemented as a template.
+int Substitute(std::wstring &s, const std::wstring &sFind, const std::wstring &sReplace);
+int Substitute(std::string &s, const std::string &sFind, const std::string &sReplace);
 
 std::string StdStringFromInteger(int i);
 
+// Basic case lowering that converts A-Z to a-z.
+// Does not handle non-ASCII characters.
+void LowerCaseAZ(std::string &s);
+
 // StringSplit can be expanded over std::string or GUI::gui_string
 template <typename T>
 std::vector<T> StringSplit(const T &text, int separator) {
@@ -31,7 +38,8 @@ inline std::vector<GUI::gui_string> ListFromString(const GUI::gui_string &args)
 
 char *Slash(const char *s, bool quoteQuotes);
 unsigned int UnSlash(char *s);
-unsigned int UnSlashLowOctal(char *s);
+std::string UnSlashString(const char *s);
+std::string UnSlashLowOctalString(const char *s);
 
 class ILocalize {
 public:
