@@ -48,44 +48,46 @@ static int CompareNCaseInsensitive(const char *a, const char *b, size_t len) {
  * Creates an array that points into each word in the string and puts \0 terminators
  * after each word.
  */
-static char **ArrayFromStringList(char *StringList, int *len, bool onlyLineEnds = false) {
+static char **ArrayFromStringList(char *stringList, int *len, bool onlyLineEnds = false) {
 	int prev = '\n';
 	int words = 0;
 	// For rapid determination of whether a character is a separator, build
 	// a look up table.
 	bool wordSeparator[256];
-	for (int i=0;i<256; i++) {
+	for (int i=0; i<256; i++) {
 		wordSeparator[i] = false;
 	}
-	wordSeparator['\r'] = true;
-	wordSeparator['\n'] = true;
+	wordSeparator[static_cast<unsigned int>('\r')] = true;
+	wordSeparator[static_cast<unsigned int>('\n')] = true;
 	if (!onlyLineEnds) {
-		wordSeparator[' '] = true;
-		wordSeparator['\t'] = true;
+		wordSeparator[static_cast<unsigned int>(' ')] = true;
+		wordSeparator[static_cast<unsigned int>('\t')] = true;
 	}
-	for (int j = 0; StringList[j]; j++) {
-		int curr = static_cast<unsigned char>(StringList[j]);
+	for (int j = 0; stringList[j]; j++) {
+		int curr = static_cast<unsigned char>(stringList[j]);
 		if (!wordSeparator[curr] && wordSeparator[prev])
 			words++;
 		prev = curr;
 	}
 	char **keywords = new char *[words + 1];
-	words = 0;
-	prev = '\0';
-	size_t slen = strlen(StringList);
-	for (size_t k = 0; k < slen; k++) {
-		if (!wordSeparator[static_cast<unsigned char>(StringList[k])]) {
-			if (!prev) {
-				keywords[words] = &StringList[k];
-				words++;
+	int wordsStore = 0;
+	const size_t slen = strlen(stringList);
+	if (words) {
+		prev = '\0';
+		for (size_t k = 0; k < slen; k++) {
+			if (!wordSeparator[static_cast<unsigned char>(stringList[k])]) {
+				if (!prev) {
+					keywords[wordsStore] = &stringList[k];
+					wordsStore++;
+				}
+			} else {
+				stringList[k] = '\0';
 			}
-		} else {
-			StringList[k] = '\0';
+			prev = stringList[k];
 		}
-		prev = StringList[k];
 	}
-	keywords[words] = &StringList[slen];
-	*len = words;
+	keywords[wordsStore] = &stringList[slen];
+	*len = wordsStore;
 	return keywords;
 }
 
@@ -146,53 +148,6 @@ static void SortStringListNoCase(char **wordsNoCase, unsigned int len) {
 	      slCmpStringNoCase);
 }
 
-bool StringList::InList(const char *s) {
-	if (0 == words)
-		return false;
-	if (!sorted) {
-		sorted = true;
-		SortStringList(words, len);
-		for (unsigned int k = 0; k < (sizeof(starts) / sizeof(starts[0])); k++)
-			starts[k] = -1;
-		for (int l = len - 1; l >= 0; l--) {
-			unsigned char indexChar = words[l][0];
-			starts[indexChar] = l;
-		}
-	}
-	unsigned char firstChar = s[0];
-	int j = starts[firstChar];
-	if (j >= 0) {
-		while ((unsigned char)words[j][0] == firstChar) {
-			if (s[1] == words[j][1]) {
-				const char *a = words[j] + 1;
-				const char *b = s + 1;
-				while (*a && *a == *b) {
-					a++;
-					b++;
-				}
-				if (!*a && !*b)
-					return true;
-			}
-			j++;
-		}
-	}
-	j = starts['^'];
-	if (j >= 0) {
-		while (words[j][0] == '^') {
-			const char *a = words[j] + 1;
-			const char *b = s;
-			while (*a && *a == *b) {
-				a++;
-				b++;
-			}
-			if (!*a)
-				return true;
-			j++;
-		}
-	}
-	return false;
-}
-
 /**
  * Returns an element (complete) of the StringList array which has
  * the same beginning as the passed string.
@@ -410,8 +365,3 @@ char *StringList::GetNearestWords(
 	}
 	return NULL;
 }
-
-#ifdef _MSC_VER
-// Unreferenced inline functions are OK
-#pragma warning(disable: 4514)
-#endif
