@@ -11,6 +11,15 @@
 #include "GUI.h"
 #include "StyleWriter.h"
 
+TextReader::TextReader(GUI::ScintillaWindow &sw_) :
+	startPos(extremePosition),
+	endPos(0),
+	codePage(0),
+	sw(sw_),
+	lenDoc(-1) {
+	buf[0] = 0;
+}
+
 bool TextReader::InternalIsLeadByte(char ch) const {
 	return GUI::IsDBCSLeadByte(codePage, ch);
 }
@@ -67,6 +76,13 @@ int TextReader::GetLineState(int line) {
 	return sw.Call(SCI_GETLINESTATE, line);
 }
 
+StyleWriter::StyleWriter(GUI::ScintillaWindow &sw_) :
+	TextReader(sw_),
+	validLen(0),
+	startSeg(0) {
+	styleBuf[0] = 0;
+}
+
 int StyleWriter::SetLineState(int line, int state) {
 	return sw.Call(SCI_SETLINESTATE, line, state);
 }
