@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+# Requires Python 2.7 or later
 
 from __future__ import with_statement
 from __future__ import unicode_literals
@@ -56,10 +57,6 @@ class TestSimple(unittest.TestCase):
 		self.ed.StartStyling(0,0xf)
 		self.ed.SetStyling(1, 5)
 		self.assertEquals(self.ed.StyledTextRange(0, 2), b"x\005y\003")
-		# Set the mask so 0 bit changed but not 2 bit
-		self.ed.StartStyling(0,0x1)
-		self.ed.SetStyling(1, 0)
-		self.assertEquals(self.ed.StyledTextRange(0, 2), b"x\004y\003")
 
 		self.ed.StartStyling(0,0xff)
 		self.ed.SetStylingEx(2, b"\100\101")
@@ -86,6 +83,12 @@ class TestSimple(unittest.TestCase):
 		self.assertEquals(self.ed.Anchor, 0)
 		self.assertEquals(self.ed.CurrentPos, 1)
 
+	def testBeyonEnd(self):
+		self.ed.AddText(1, b"x")
+		self.assertEquals(self.ed.GetLineEndPosition(0), 1)
+		self.assertEquals(self.ed.GetLineEndPosition(1), 1)
+		self.assertEquals(self.ed.GetLineEndPosition(2), 1)
+
 	def testSelection(self):
 		self.assertEquals(self.ed.CurrentPos, 0)
 		self.assertEquals(self.ed.Anchor, 0)
@@ -284,7 +287,7 @@ class TestSimple(unittest.TestCase):
 			self.assertEquals(self.ed.LineLength(0), 1 + len(lineEnds[lineEndType]))
 
 	# Several tests for unicode line ends U+2028 and U+2029
-	
+
 	def testUnicodeLineEnds(self):
 		# Add two lines separated with U+2028 and ensure it is seen as two lines
 		# Then remove U+2028 and should be just 1 lines
@@ -303,6 +306,7 @@ class TestSimple(unittest.TestCase):
 		self.assertEquals(self.ed.LineCount, 1)
 		self.assertEquals(self.ed.LineLength(0), 2)
 		self.assertEquals(self.ed.GetLineEndPosition(0), 2)
+		self.assertEquals(self.ed.LineEndTypesSupported, 1)
 
 	def testUnicodeLineEndsWithCodePage0(self):
 		# Try the Unicode line ends when not in Unicode mode -> should remain 1 line
@@ -368,7 +372,7 @@ class TestSimple(unittest.TestCase):
 		text = b"x\xe2\x80\xa9y";
 		self.ed.AddText(5, text)
 		self.assertEquals(self.ed.LineCount, 2)
-		
+
 		for i in range(len(text)):
 			self.ed.TargetStart = i
 			self.ed.TargetEnd = i + 1
@@ -379,7 +383,7 @@ class TestSimple(unittest.TestCase):
 			else:
 				# Removing byte from line end, removes 1 line
 				self.assertEquals(self.ed.LineCount, 1)
-				
+
 			self.ed.TargetEnd = i
 			self.ed.ReplaceTarget(1, text[i:i+1])
 			self.assertEquals(self.ed.LineCount, 2)
@@ -397,7 +401,7 @@ class TestSimple(unittest.TestCase):
 			self.ed.TargetEnd = i+2
 			self.ed.ReplaceTarget(0, b"")
 			self.assertEquals(self.ed.LineCount, 1)
-		
+
 	# Several tests for unicode NEL line ends U+0085
 
 	def testNELLineEnds(self):
@@ -450,7 +454,7 @@ class TestSimple(unittest.TestCase):
 		text = b"x\xc2\x85y";
 		self.ed.AddText(4, text)
 		self.assertEquals(self.ed.LineCount, 2)
-		
+
 		for i in range(len(text)):
 			self.ed.TargetStart = i
 			self.ed.TargetEnd = i + 1
@@ -461,7 +465,7 @@ class TestSimple(unittest.TestCase):
 			else:
 				# Removing byte from line end, removes 1 line
 				self.assertEquals(self.ed.LineCount, 1)
-				
+
 			self.ed.TargetEnd = i
 			self.ed.ReplaceTarget(1, text[i:i+1])
 			self.assertEquals(self.ed.LineCount, 2)
@@ -477,7 +481,7 @@ class TestSimple(unittest.TestCase):
 			self.ed.TargetEnd = i+2
 			self.ed.ReplaceTarget(0, b"")
 			self.assertEquals(self.ed.LineCount, 1)
-		
+
 	def testGoto(self):
 		self.ed.AddText(5, b"a\nb\nc")
 		self.assertEquals(self.ed.CurrentPos, 5)
@@ -1376,6 +1380,123 @@ class TestMultiSelection(unittest.TestCase):
 		self.assertEquals(self.ed.GetSelectionNCaret(0), 3)
 		self.assertEquals(self.ed.GetSelectionNCaretVirtualSpace(0), 0)
 
+	def testDropSelectionN(self):
+		self.ed.SetSelection(1, 2)
+		# Only one so dropping has no effect
+		self.ed.DropSelectionN(0)
+		self.assertEquals(self.ed.Selections, 1)
+		self.ed.AddSelection(4, 5)
+		self.assertEquals(self.ed.Selections, 2)
+		# Outside bounds so no effect
+		self.ed.DropSelectionN(2)
+		self.assertEquals(self.ed.Selections, 2)
+		# Dropping before main so main decreases
+		self.ed.DropSelectionN(0)
+		self.assertEquals(self.ed.Selections, 1)
+		self.assertEquals(self.ed.MainSelection, 0)
+		self.assertEquals(self.ed.GetSelectionNCaret(0), 4)
+		self.assertEquals(self.ed.GetSelectionNAnchor(0), 5)
+
+		self.ed.AddSelection(10, 11)
+		self.ed.AddSelection(20, 21)
+		self.assertEquals(self.ed.Selections, 3)
+		self.assertEquals(self.ed.MainSelection, 2)
+		self.ed.MainSelection = 1
+		# Dropping after main so main does not change
+		self.ed.DropSelectionN(2)
+		self.assertEquals(self.ed.MainSelection, 1)
+
+		# Dropping first selection so wraps around to new last.
+		self.ed.AddSelection(30, 31)
+		self.ed.AddSelection(40, 41)
+		self.assertEquals(self.ed.Selections, 4)
+		self.ed.MainSelection = 0
+		self.ed.DropSelectionN(0)
+		self.assertEquals(self.ed.MainSelection, 2)
+
+class TestStyleAttributes(unittest.TestCase):
+	""" These tests are just to ensure that the calls set and retrieve values.
+	They do not check the visual appearance of the style attributes.
+	"""
+	def setUp(self):
+		self.xite = Xite.xiteFrame
+		self.ed = self.xite.ed
+		self.ed.ClearAll()
+		self.ed.EmptyUndoBuffer()
+		self.testColour = 0x171615
+		self.testFont = b"Georgia"
+
+	def tearDown(self):
+		self.ed.StyleResetDefault()
+
+	def testFont(self):
+		self.ed.StyleSetFont(self.ed.STYLE_DEFAULT, self.testFont)
+		self.assertEquals(self.ed.StyleGetFont(self.ed.STYLE_DEFAULT), self.testFont)
+
+	def testSize(self):
+		self.ed.StyleSetSize(self.ed.STYLE_DEFAULT, 12)
+		self.assertEquals(self.ed.StyleGetSize(self.ed.STYLE_DEFAULT), 12)
+		self.assertEquals(self.ed.StyleGetSizeFractional(self.ed.STYLE_DEFAULT), 12*self.ed.SC_FONT_SIZE_MULTIPLIER)
+		self.ed.StyleSetSizeFractional(self.ed.STYLE_DEFAULT, 1234)
+		self.assertEquals(self.ed.StyleGetSizeFractional(self.ed.STYLE_DEFAULT), 1234)
+
+	def testBold(self):
+		self.ed.StyleSetBold(self.ed.STYLE_DEFAULT, 1)
+		self.assertEquals(self.ed.StyleGetBold(self.ed.STYLE_DEFAULT), 1)
+		self.assertEquals(self.ed.StyleGetWeight(self.ed.STYLE_DEFAULT), self.ed.SC_WEIGHT_BOLD)
+		self.ed.StyleSetWeight(self.ed.STYLE_DEFAULT, 530)
+		self.assertEquals(self.ed.StyleGetWeight(self.ed.STYLE_DEFAULT), 530)
+
+	def testItalic(self):
+		self.ed.StyleSetItalic(self.ed.STYLE_DEFAULT, 1)
+		self.assertEquals(self.ed.StyleGetItalic(self.ed.STYLE_DEFAULT), 1)
+
+	def testUnderline(self):
+		self.assertEquals(self.ed.StyleGetUnderline(self.ed.STYLE_DEFAULT), 0)
+		self.ed.StyleSetUnderline(self.ed.STYLE_DEFAULT, 1)
+		self.assertEquals(self.ed.StyleGetUnderline(self.ed.STYLE_DEFAULT), 1)
+
+	def testFore(self):
+		self.assertEquals(self.ed.StyleGetFore(self.ed.STYLE_DEFAULT), 0)
+		self.ed.StyleSetFore(self.ed.STYLE_DEFAULT, self.testColour)
+		self.assertEquals(self.ed.StyleGetFore(self.ed.STYLE_DEFAULT), self.testColour)
+
+	def testBack(self):
+		self.assertEquals(self.ed.StyleGetBack(self.ed.STYLE_DEFAULT), 0xffffff)
+		self.ed.StyleSetBack(self.ed.STYLE_DEFAULT, self.testColour)
+		self.assertEquals(self.ed.StyleGetBack(self.ed.STYLE_DEFAULT), self.testColour)
+
+	def testEOLFilled(self):
+		self.assertEquals(self.ed.StyleGetEOLFilled(self.ed.STYLE_DEFAULT), 0)
+		self.ed.StyleSetEOLFilled(self.ed.STYLE_DEFAULT, 1)
+		self.assertEquals(self.ed.StyleGetEOLFilled(self.ed.STYLE_DEFAULT), 1)
+
+	def testCharacterSet(self):
+		self.ed.StyleSetCharacterSet(self.ed.STYLE_DEFAULT, self.ed.SC_CHARSET_RUSSIAN)
+		self.assertEquals(self.ed.StyleGetCharacterSet(self.ed.STYLE_DEFAULT), self.ed.SC_CHARSET_RUSSIAN)
+
+	def testCase(self):
+		self.assertEquals(self.ed.StyleGetCase(self.ed.STYLE_DEFAULT), self.ed.SC_CASE_MIXED)
+		self.ed.StyleSetCase(self.ed.STYLE_DEFAULT, self.ed.SC_CASE_UPPER)
+		self.assertEquals(self.ed.StyleGetCase(self.ed.STYLE_DEFAULT), self.ed.SC_CASE_UPPER)
+		self.ed.StyleSetCase(self.ed.STYLE_DEFAULT, self.ed.SC_CASE_LOWER)
+		self.assertEquals(self.ed.StyleGetCase(self.ed.STYLE_DEFAULT), self.ed.SC_CASE_LOWER)
+
+	def testVisible(self):
+		self.assertEquals(self.ed.StyleGetVisible(self.ed.STYLE_DEFAULT), 1)
+		self.ed.StyleSetVisible(self.ed.STYLE_DEFAULT, 0)
+		self.assertEquals(self.ed.StyleGetVisible(self.ed.STYLE_DEFAULT), 0)
+
+	def testChangeable(self):
+		self.assertEquals(self.ed.StyleGetChangeable(self.ed.STYLE_DEFAULT), 1)
+		self.ed.StyleSetChangeable(self.ed.STYLE_DEFAULT, 0)
+		self.assertEquals(self.ed.StyleGetChangeable(self.ed.STYLE_DEFAULT), 0)
+
+	def testHotSpot(self):
+		self.assertEquals(self.ed.StyleGetHotSpot(self.ed.STYLE_DEFAULT), 0)
+		self.ed.StyleSetHotSpot(self.ed.STYLE_DEFAULT, 1)
+		self.assertEquals(self.ed.StyleGetHotSpot(self.ed.STYLE_DEFAULT), 1)
+
 class TestCharacterNavigation(unittest.TestCase):
 	def setUp(self):
 		self.xite = Xite.xiteFrame
@@ -1489,7 +1610,7 @@ class TestCaseMapping(unittest.TestCase):
 	def testUTFGrows(self):
 		# This crashed at one point in debug builds due to looking past end of shorter string
 		self.ed.SetCodePage(65001)
-		# ﬖ is a single character ligature taking 3 bytes in UTF8: EF AC 96 
+		# ﬖ is a single character ligature taking 3 bytes in UTF8: EF AC 96
 		t = 'ﬖﬖ'.encode("UTF-8")
 		self.ed.SetContents(t)
 		self.assertEquals(self.ed.Length, 6)
@@ -1612,7 +1733,7 @@ class TestLexer(unittest.TestCase):
 		self.assertEquals(self.ed.GetLexer(), self.ed.SCLEX_CPP)
 		name = self.ed.GetLexerLanguage(0)
 		self.assertEquals(name, b"cpp")
-	
+
 	def testPropertyNames(self):
 		propertyNames = self.ed.PropertyNames()
 		self.assertNotEquals(propertyNames, b"")
@@ -1627,6 +1748,78 @@ class TestLexer(unittest.TestCase):
 		wordSet = self.ed.DescribeKeyWordSets()
 		self.assertNotEquals(wordSet, b"")
 
+class TestSubStyles(unittest.TestCase):
+	''' These tests include knowledge of the current implementation in the cpp lexer
+	and may have to change when that implementation changes.
+	Currently supports subStyles for IDENTIFIER 11 and COMMENTDOCKEYWORD 17 '''
+	def setUp(self):
+		self.xite = Xite.xiteFrame
+		self.ed = self.xite.ed
+		self.ed.ClearAll()
+		self.ed.EmptyUndoBuffer()
+
+	def testInfo(self):
+		self.ed.Lexer = self.ed.SCLEX_CPP
+		bases = self.ed.GetSubStyleBases()
+		self.assertEquals(bases, b"\x0b\x11")	# 11, 17
+		self.assertEquals(self.ed.DistanceToSecondaryStyles(), 0x40)
+
+	def testAllocate(self):
+		firstSubStyle = 0x80	# Current implementation
+		self.ed.Lexer = self.ed.SCLEX_CPP
+		self.assertEquals(self.ed.GetStyleFromSubStyle(firstSubStyle), firstSubStyle)
+		self.assertEquals(self.ed.GetSubStylesStart(self.ed.SCE_C_IDENTIFIER), 0)
+		self.assertEquals(self.ed.GetSubStylesLength(self.ed.SCE_C_IDENTIFIER), 0)
+		numSubStyles = 5
+		subs = self.ed.AllocateSubStyles(self.ed.SCE_C_IDENTIFIER, numSubStyles)
+		self.assertEquals(subs, firstSubStyle)
+		self.assertEquals(self.ed.GetSubStylesStart(self.ed.SCE_C_IDENTIFIER), firstSubStyle)
+		self.assertEquals(self.ed.GetSubStylesLength(self.ed.SCE_C_IDENTIFIER), numSubStyles)
+		self.assertEquals(self.ed.GetStyleFromSubStyle(subs), self.ed.SCE_C_IDENTIFIER)
+		self.assertEquals(self.ed.GetStyleFromSubStyle(subs+numSubStyles-1), self.ed.SCE_C_IDENTIFIER)
+		self.assertEquals(self.ed.GetStyleFromSubStyle(self.ed.SCE_C_IDENTIFIER), self.ed.SCE_C_IDENTIFIER)
+		# Now free and check same as start
+		self.ed.FreeSubStyles()
+		self.assertEquals(self.ed.GetStyleFromSubStyle(subs), subs)
+		self.assertEquals(self.ed.GetSubStylesStart(self.ed.SCE_C_IDENTIFIER), 0)
+		self.assertEquals(self.ed.GetSubStylesLength(self.ed.SCE_C_IDENTIFIER), 0)
+
+	def testInactive(self):
+		firstSubStyle = 0x80	# Current implementation
+		inactiveDistance = self.ed.DistanceToSecondaryStyles()
+		self.ed.Lexer = self.ed.SCLEX_CPP
+		numSubStyles = 5
+		subs = self.ed.AllocateSubStyles(self.ed.SCE_C_IDENTIFIER, numSubStyles)
+		self.assertEquals(subs, firstSubStyle)
+		self.assertEquals(self.ed.GetStyleFromSubStyle(subs), self.ed.SCE_C_IDENTIFIER)
+		self.assertEquals(self.ed.GetStyleFromSubStyle(subs+inactiveDistance), self.ed.SCE_C_IDENTIFIER+inactiveDistance)
+		self.ed.FreeSubStyles()
+
+	def testSecondary(self):
+		inactiveDistance = self.ed.DistanceToSecondaryStyles()
+		self.assertEquals(self.ed.GetPrimaryStyleFromStyle(self.ed.SCE_C_IDENTIFIER+inactiveDistance), self.ed.SCE_C_IDENTIFIER)
+
+class TestCallTip(unittest.TestCase):
+
+	def setUp(self):
+		self.xite = Xite.xiteFrame
+		self.ed = self.xite.ed
+		self.ed.ClearAll()
+		self.ed.EmptyUndoBuffer()
+		# 1 line of 4 characters
+		t = b"fun("
+		self.ed.AddText(len(t), t)
+
+	def testBasics(self):
+		self.assertEquals(self.ed.CallTipActive(), 0)
+		self.ed.CallTipShow(1, "fun(int x)")
+		self.assertEquals(self.ed.CallTipActive(), 1)
+		self.assertEquals(self.ed.CallTipPosStart(), 4)
+		self.ed.CallTipSetPosStart(1)
+		self.assertEquals(self.ed.CallTipPosStart(), 1)
+		self.ed.CallTipCancel()
+		self.assertEquals(self.ed.CallTipActive(), 0)
+
 class TestAutoComplete(unittest.TestCase):
 
 	def setUp(self):
@@ -1712,6 +1905,12 @@ class TestAutoComplete(unittest.TestCase):
 
 		self.assertEquals(self.ed.AutoCActive(), 0)
 
+	def testWriteOnly(self):
+		""" Checks that setting attributes doesn't crash or change tested behaviour
+		but does not check that the changed attributes are effective. """
+		self.ed.AutoCStops(0, b"abcde")
+		self.ed.AutoCSetFillUps(0, b"1234")
+
 class TestDirectAccess(unittest.TestCase):
 
 	def setUp(self):
@@ -1843,6 +2042,80 @@ class TestWordChars(unittest.TestCase):
 		data = self.ed.GetPunctuationChars(None)
 		self.assertCharSetsEqual(data, expected)
 
+class TestExplicitTabStops(unittest.TestCase):
+
+	def setUp(self):
+		self.xite = Xite.xiteFrame
+		self.ed = self.xite.ed
+		self.ed.ClearAll()
+		self.ed.EmptyUndoBuffer()
+		# 2 lines of 4 characters
+		self.t = b"fun(\nint)"
+		self.ed.AddText(len(self.t), self.t)
+
+	def testAddingAndClearing(self):
+		self.assertEquals(self.ed.GetNextTabStop(0,0), 0)
+
+		# Add a tab stop at 7
+		self.ed.AddTabStop(0, 7)
+		# Check added
+		self.assertEquals(self.ed.GetNextTabStop(0,0), 7)
+		# Check does not affect line 1
+		self.assertEquals(self.ed.GetNextTabStop(1,0), 0)
+
+		# Add a tab stop at 18
+		self.ed.AddTabStop(0, 18)
+		# Check added
+		self.assertEquals(self.ed.GetNextTabStop(0,0), 7)
+		self.assertEquals(self.ed.GetNextTabStop(0,7), 18)
+		# Check does not affect line 1
+		self.assertEquals(self.ed.GetNextTabStop(1,0), 0)
+		self.assertEquals(self.ed.GetNextTabStop(1,7), 0)
+
+		# Add a tab stop between others at 13
+		self.ed.AddTabStop(0, 13)
+		# Check added
+		self.assertEquals(self.ed.GetNextTabStop(0,0), 7)
+		self.assertEquals(self.ed.GetNextTabStop(0,7), 13)
+		self.assertEquals(self.ed.GetNextTabStop(0,13), 18)
+		# Check does not affect line 1
+		self.assertEquals(self.ed.GetNextTabStop(1,0), 0)
+		self.assertEquals(self.ed.GetNextTabStop(1,7), 0)
+
+		self.ed.ClearTabStops(0)
+		# Check back to original state
+		self.assertEquals(self.ed.GetNextTabStop(0,0), 0)
+
+	def testLineInsertionDeletion(self):
+		# Add a tab stop at 7 on line 1
+		self.ed.AddTabStop(1, 7)
+		# Check added
+		self.assertEquals(self.ed.GetNextTabStop(1,0), 7)
+
+		# More text at end
+		self.ed.AddText(len(self.t), self.t)
+		self.assertEquals(self.ed.GetNextTabStop(0,0), 0)
+		self.assertEquals(self.ed.GetNextTabStop(1,0), 7)
+		self.assertEquals(self.ed.GetNextTabStop(2,0), 0)
+		self.assertEquals(self.ed.GetNextTabStop(3,0), 0)
+
+		# Another 2 lines before explicit line moves the explicit tab stop
+		data = b"x\ny\n"
+		self.ed.InsertText(4, data)
+		self.assertEquals(self.ed.GetNextTabStop(0,0), 0)
+		self.assertEquals(self.ed.GetNextTabStop(1,0), 0)
+		self.assertEquals(self.ed.GetNextTabStop(2,0), 0)
+		self.assertEquals(self.ed.GetNextTabStop(3,0), 7)
+		self.assertEquals(self.ed.GetNextTabStop(4,0), 0)
+		self.assertEquals(self.ed.GetNextTabStop(5,0), 0)
+
+		# Undo moves the explicit tab stop back
+		self.ed.Undo()
+		self.assertEquals(self.ed.GetNextTabStop(0,0), 0)
+		self.assertEquals(self.ed.GetNextTabStop(1,0), 7)
+		self.assertEquals(self.ed.GetNextTabStop(2,0), 0)
+		self.assertEquals(self.ed.GetNextTabStop(3,0), 0)
+
 if __name__ == '__main__':
 	uu = Xite.main("simpleTests")
 	#~ for x in sorted(uu.keys()):
