@@ -167,6 +167,19 @@ gui_string StringFromInteger(long i) {
 	return gui_string(gnumber);
 }
 
+gui_string HexStringFromInteger(long i) {
+	char number[32];
+	sprintf(number, "%0lx", i);
+	gui_char gnumber[32];
+	size_t n = 0;
+	while (number[n]) {
+		gnumber[n] = static_cast<gui_char>(number[n]);
+		n++;
+	}
+	gnumber[n] = 0;
+	return gui_string(gnumber);
+}
+
 void Window::Destroy() {
 	if (wid)
 		::DestroyWindow(reinterpret_cast<HWND>(wid));
