@@ -8,10 +8,10 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
-#include <ctype.h>
 #include <stdarg.h>
 #include <time.h>
 #include <math.h>
+#include <ctype.h>
 #include <limits.h>
 
 #include <vector>
@@ -1547,9 +1547,9 @@ void SurfaceD2D::DrawRGBAImage(PRectangle rc, int width, int height, const unsig
 		rc.bottom = rc.top + height;
 
 		std::vector<unsigned char> image(height * width * 4);
-		for (int y=0; y<height; y++) {
-			for (int x=0; x<width; x++) {
-				unsigned char *pixel = &image[0] + (y*width+x) * 4;
+		for (int yPixel=0; yPixel<height; yPixel++) {
+			for (int xPixel = 0; xPixel<width; xPixel++) {
+				unsigned char *pixel = &image[0] + (yPixel*width + xPixel) * 4;
 				unsigned char alpha = pixelsImage[3];
 				// Input is RGBA, output is BGRA with premultiplied alpha
 				pixel[2] = (*pixelsImage++) * alpha / 255;
@@ -2180,7 +2180,7 @@ public:
 		}
 	}
 	virtual void SetFont(Font &font);
-	virtual void Create(Window &parent, int ctrlID, Point location_, int lineHeight_, bool unicodeMode_, int technology_);
+	virtual void Create(Window &parent_, int ctrlID_, Point location_, int lineHeight_, bool unicodeMode_, int technology_);
 	virtual void SetAverageCharWidth(int width);
 	virtual void SetVisibleRows(int rows);
 	virtual int GetVisibleRows() const;
@@ -3280,31 +3280,33 @@ void Platform_Initialise(void *hInstance) {
 #pragma warning(default: 4996)
 #endif
 
-void Platform_Finalise() {
+void Platform_Finalise(bool fromDllMain) {
 #if defined(USE_D2D)
-	if (defaultRenderingParams) {
-		defaultRenderingParams->Release();
-		defaultRenderingParams = 0;
-	}
-	if (customClearTypeRenderingParams) {
-		customClearTypeRenderingParams->Release();
-		customClearTypeRenderingParams = 0;
-	}
-	if (pIDWriteFactory) {
-		pIDWriteFactory->Release();
-		pIDWriteFactory = 0;
-	}
-	if (pD2DFactory) {
-		pD2DFactory->Release();
-		pD2DFactory = 0;
-	}
-	if (hDLLDWrite) {
-		FreeLibrary(hDLLDWrite);
-		hDLLDWrite = NULL;
-	}
-	if (hDLLD2D) {
-		FreeLibrary(hDLLD2D);
-		hDLLD2D = NULL;
+	if (!fromDllMain) {
+		if (defaultRenderingParams) {
+			defaultRenderingParams->Release();
+			defaultRenderingParams = 0;
+		}
+		if (customClearTypeRenderingParams) {
+			customClearTypeRenderingParams->Release();
+			customClearTypeRenderingParams = 0;
+		}
+		if (pIDWriteFactory) {
+			pIDWriteFactory->Release();
+			pIDWriteFactory = 0;
+		}
+		if (pD2DFactory) {
+			pD2DFactory->Release();
+			pD2DFactory = 0;
+		}
+		if (hDLLDWrite) {
+			FreeLibrary(hDLLDWrite);
+			hDLLDWrite = NULL;
+		}
+		if (hDLLD2D) {
+			FreeLibrary(hDLLD2D);
+			hDLLD2D = NULL;
+		}
 	}
 #endif
 	if (reverseArrowCursor != NULL)
