@@ -187,34 +187,43 @@ int SciTEWin::DoDialog(HINSTANCE hInst, const TCHAR *resName, HWND hWnd, DLGPROC
 
 
 GUI::gui_string SciTEWin::DialogFilterFromProperty(const GUI::gui_char *filterProperty) {
-	GUI::gui_string filter = filterProperty;
-	if (filter.length()) {
-		std::replace(filter.begin(), filter.end(), '|', '\0');
+	GUI::gui_string filterText = filterProperty;
+	if (filterText.length()) {
+		std::replace(filterText.begin(), filterText.end(), '|', '\0');
 		size_t start = 0;
-		while (start < filter.length()) {
-			const GUI::gui_char *filterName = filter.c_str() + start;
+		while (start < filterText.length()) {
+			const GUI::gui_char *filterName = filterText.c_str() + start;
 			if (*filterName == '#') {
-				size_t next = start + wcslen(filter.c_str() + start) + 1;
-				next += wcslen(filter.c_str() + next) + 1;
-				filter.erase(start, next - start);
+				size_t next = start + wcslen(filterText.c_str() + start) + 1;
+				next += wcslen(filterText.c_str() + next) + 1;
+				filterText.erase(start, next - start);
 			} else {
 				GUI::gui_string localised = localiser.Text(GUI::UTF8FromString(filterName).c_str(), false);
 				if (localised.size()) {
-					filter.erase(start, wcslen(filterName));
-					filter.insert(start, localised.c_str());
+					filterText.erase(start, wcslen(filterName));
+					filterText.insert(start, localised.c_str());
 				}
-				start += wcslen(filter.c_str() + start) + 1;
-				start += wcslen(filter.c_str() + start) + 1;
+				start += wcslen(filterText.c_str() + start) + 1;
+				start += wcslen(filterText.c_str() + start) + 1;
 			}
 		}
 	}
-	return filter;
+	return filterText;
 }
 
-bool SciTEWin::OpenDialog(FilePath directory, const GUI::gui_char *filter) {
+void SciTEWin::CheckCommonDialogError() {
+	DWORD errorNumber = ::CommDlgExtendedError();
+	if (errorNumber) {
+		GUI::gui_string sError = GUI::HexStringFromInteger(errorNumber);
+		GUI::gui_string msg = LocaliseMessage("Common dialog error 0x^0.", sError.c_str());
+		WindowMessageBox(wSciTE, msg);
+	}
+}
+
+bool SciTEWin::OpenDialog(FilePath directory, const GUI::gui_char *filesFilter) {
 	enum {maxBufferSize=2048};
 
-	GUI::gui_string openFilter = DialogFilterFromProperty(filter);
+	GUI::gui_string openFilter = DialogFilterFromProperty(filesFilter);
 
 	if (!openWhat[0]) {
 		StringCopy(openWhat, localiser.Text("Custom Filter").c_str());
@@ -268,11 +277,13 @@ bool SciTEWin::OpenDialog(FilePath directory, const GUI::gui_char *filter) {
 				p += wcslen(p) + 1;
 			}
 		}
+	} else {
+		CheckCommonDialogError();
 	}
 	return succeeded;
 }
 
-FilePath SciTEWin::ChooseSaveName(FilePath directory, const char *title, const GUI::gui_char *filter, const char *ext) {
+FilePath SciTEWin::ChooseSaveName(FilePath directory, const char *title, const GUI::gui_char *filesFilter, const char *ext) {
 	FilePath path;
 	if (0 == dialogsOnScreen) {
 		GUI::gui_char saveName[MAX_PATH] = GUI_TEXT("");
@@ -290,12 +301,14 @@ FilePath SciTEWin::ChooseSaveName(FilePath directory, const char *title, const G
 		GUI::gui_string translatedTitle = localiser.Text(title);
 		ofn.lpstrTitle = translatedTitle.c_str();
 		ofn.Flags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_NOCHANGEDIR;
-		ofn.lpstrFilter = filter;
+		ofn.lpstrFilter = filesFilter;
 		ofn.lpstrInitialDir = directory.AsInternal();
 
 		dialogsOnScreen++;
 		if (::GetSaveFileNameW(&ofn)) {
 			path = saveName;
+		} else {
+			CheckCommonDialogError();
 		}
 		dialogsOnScreen--;
 	}
@@ -375,6 +388,8 @@ void SciTEWin::LoadSessionDialog() {
 	if (::GetOpenFileNameW(&ofn)) {
 		LoadSessionFile(openName);
 		RestoreSession();
+	} else {
+		CheckCommonDialogError();
 	}
 }
 
@@ -395,6 +410,8 @@ void SciTEWin::SaveSessionDialog() {
 	ofn.lpstrFilter = GUI_TEXT("Session (.session)\0*.session\0");
 	if (::GetSaveFileNameW(&ofn)) {
 		SaveSessionFile(saveName);
+	} else {
+		CheckCommonDialogError();
 	}
 }
 
@@ -445,6 +462,7 @@ void SciTEWin::Print(
 		pdlg.Flags |= PD_RETURNDEFAULT;
 	}
 	if (!::PrintDlg(&pdlg)) {
+		CheckCommonDialogError();
 		return;
 	}
 
@@ -583,7 +601,7 @@ void SciTEWin::Print(
 		DeleteFontObject(fontHeader);
 		DeleteFontObject(fontFooter);
 		GUI::gui_string msg = LocaliseMessage("Can not start printer document.");
-		WindowMessageBox(wSciTE, msg, MB_OK);
+		WindowMessageBox(wSciTE, msg, mbsOK);
 		return;
 	}
 
@@ -730,8 +748,10 @@ void SciTEWin::PrintSetup() {
 	pdlg.hDevMode = hDevMode;
 	pdlg.hDevNames = hDevNames;
 
-	if (!PageSetupDlg(&pdlg))
+	if (!PageSetupDlg(&pdlg)) {
+		CheckCommonDialogError();
 		return;
+	}
 
 	pagesetupMargin.left = pdlg.rtMargin.left;
 	pagesetupMargin.top = pdlg.rtMargin.top;
@@ -964,7 +984,7 @@ BOOL CALLBACK SciTEWin::FindDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM l
 	return Caller(hDlg, message, lParam)->FindMessage(hDlg, message, wParam);
 }
 
-BOOL SciTEWin::HandleReplaceCommand(int cmd, bool reverseFind) {
+BOOL SciTEWin::HandleReplaceCommand(int cmd, bool reverseDirection) {
 	if (!wFindReplace.GetID())
 		return TRUE;
 	HWND hwndFR = reinterpret_cast<HWND>(wFindReplace.GetID());
@@ -977,7 +997,7 @@ BOOL SciTEWin::HandleReplaceCommand(int cmd, bool reverseFind) {
 
 	int replacements = 0;
 	if (cmd == IDOK) {
-		FindNext(reverseFind);
+		FindNext(reverseDirection);
 	} else if (cmd == IDREPLACE) {
 		ReplaceOnce();
 	} else if ((cmd == IDREPLACEALL) || (cmd == IDREPLACEINSEL)) {
@@ -1202,7 +1222,7 @@ BOOL SciTEWin::GrepMessage(HWND hDlg, UINT message, WPARAM wParam) {
 		} else if (ControlIDOfWParam(wParam) == IDOK) {
 			if (jobQueue.IsExecuting()) {
 				GUI::gui_string msgBuf = LocaliseMessage("Job is currently executing. Wait until it finishes.");
-				WindowMessageBox(wFindInFiles, msgBuf, MB_OK | MB_ICONWARNING);
+				WindowMessageBox(wFindInFiles, msgBuf);
 				return FALSE;
 			}
 			findWhat = dlg.ItemTextU(IDFINDWHAT).string();
@@ -1635,21 +1655,32 @@ bool SciTEWin::ParametersDialog(bool modal) {
 	return success;
 }
 
-int SciTEWin::WindowMessageBox(GUI::Window &w, const GUI::gui_string &msg, int style) {
+SciTEBase::MessageBoxChoice SciTEWin::WindowMessageBox(GUI::Window &w, const GUI::gui_string &msg, MessageBoxStyle style) {
 	dialogsOnScreen++;
 	int ret = ::MessageBoxW(reinterpret_cast<HWND>(w.GetID()), msg.c_str(), appName, style | MB_SETFOREGROUND);
 	dialogsOnScreen--;
-	return ret;
+	switch (ret) {
+	case IDOK:
+		return mbOK;
+	case IDCANCEL:
+		return mbCancel;
+	case IDYES:
+		return mbYes;
+	case IDNO:
+		return mbNo;
+	default:
+		return mbOK;
+	}
 }
 
 void SciTEWin::FindMessageBox(const SString &msg, const std::string *findItem) {
 	if (findItem == 0) {
 		GUI::gui_string msgBuf = LocaliseMessage(msg.c_str());
-		WindowMessageBox(wFindReplace.Created() ? wFindReplace : wSciTE, msgBuf, MB_OK | MB_ICONWARNING);
+		WindowMessageBox(wFindReplace.Created() ? wFindReplace : wSciTE, msgBuf);
 	} else {
 		GUI::gui_string findThing = GUI::StringFromUTF8(findItem->c_str());
 		GUI::gui_string msgBuf = LocaliseMessage(msg.c_str(), findThing.c_str());
-		WindowMessageBox(wFindReplace.Created() ? wFindReplace : wSciTE, msgBuf, MB_OK | MB_ICONWARNING);
+		WindowMessageBox(wFindReplace.Created() ? wFindReplace : wSciTE, msgBuf);
 	}
 }
 
