@@ -8,8 +8,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
-#include <ctype.h>
 #include <assert.h>
+#include <ctype.h>
 #include <limits.h>
 
 #include <new>
@@ -26,6 +26,8 @@
 #include <commctrl.h>
 #include <richedit.h>
 #include <windowsx.h>
+#include <zmouse.h>
+#include <ole2.h>
 
 #if defined(NTDDI_WIN7) && !defined(DISABLE_D2D)
 #define USE_D2D 1
@@ -43,9 +45,11 @@
 
 #ifdef SCI_LEXER
 #include "SciLexer.h"
-#include "LexerModule.h"
 #endif
 #include "StringCopy.h"
+#ifdef SCI_LEXER
+#include "LexerModule.h"
+#endif
 #include "SplitVector.h"
 #include "Partitioning.h"
 #include "RunStyles.h"
@@ -57,25 +61,29 @@
 #include "XPM.h"
 #include "LineMarker.h"
 #include "Style.h"
-#include "AutoComplete.h"
 #include "ViewStyle.h"
 #include "CharClassify.h"
 #include "Decoration.h"
 #include "CaseFolder.h"
 #include "Document.h"
+#include "CaseConvert.h"
+#include "UniConversion.h"
 #include "Selection.h"
 #include "PositionCache.h"
+#include "EditModel.h"
+#include "MarginView.h"
+#include "EditView.h"
 #include "Editor.h"
-#include "ScintillaBase.h"
-#include "UniConversion.h"
-#include "CaseConvert.h"
 
-#include "PlatWin.h"
+#include "AutoComplete.h"
+#include "ScintillaBase.h"
 
 #ifdef SCI_LEXER
 #include "ExternalLexer.h"
 #endif
 
+#include "PlatWin.h"
+
 #ifndef SPI_GETWHEELSCROLLLINES
 #define SPI_GETWHEELSCROLLLINES   104
 #endif
@@ -88,10 +96,6 @@
 #define UNICODE_NOCHAR                  0xFFFF
 #endif
 
-#include <commctrl.h>
-#include <zmouse.h>
-#include <ole2.h>
-
 #ifndef MK_ALT
 #define MK_ALT 32
 #endif
@@ -99,6 +103,8 @@
 #define SC_WIN_IDLE 5001
 
 typedef BOOL (WINAPI *TrackMouseEventSig)(LPTRACKMOUSEEVENT);
+typedef UINT_PTR (WINAPI *SetCoalescableTimerSig)(HWND hwnd, UINT_PTR nIDEvent,
+	UINT uElapse, TIMERPROC lpTimerFunc, ULONG uToleranceDelay);
 
 // GCC has trouble with the standard COM ABI so do it the old C way with explicit vtables.
 
@@ -179,6 +185,7 @@ class ScintillaWin :
 	bool capturedMouse;
 	bool trackedMouseLeave;
 	TrackMouseEventSig TrackMouseEventFn;
+	SetCoalescableTimerSig SetCoalescableTimerFn;
 
 	unsigned int linesPerScroll;	///< Intellimouse support
 	int wheelDelta; ///< Wheel delta from roll
@@ -223,17 +230,23 @@ class ScintillaWin :
 	static sptr_t PASCAL CTWndProc(
 		    HWND hWnd, UINT iMessage, WPARAM wParam, sptr_t lParam);
 
-	enum { invalidTimerID, standardTimerID, idleTimerID };
+	enum { invalidTimerID, standardTimerID, idleTimerID, fineTimerStart };
 
 	virtual bool DragThreshold(Point ptStart, Point ptNow);
 	virtual void StartDrag();
 	sptr_t WndPaint(uptr_t wParam);
 	sptr_t HandleComposition(uptr_t wParam, sptr_t lParam);
+	static bool KoreanIME();
+	sptr_t HandleCompositionKoreanIME(uptr_t wParam, sptr_t lParam);
 	UINT CodePageOfDocument();
 	virtual bool ValidCodePage(int codePage) const;
 	virtual sptr_t DefWndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam);
 	virtual bool SetIdle(bool on);
-	virtual void SetTicking(bool on);
+	UINT_PTR timers[tickDwell+1];
+	virtual bool FineTickerAvailable();
+	virtual bool FineTickerRunning(TickReason reason);
+	virtual void FineTickerStart(TickReason reason, int millis, int tolerance);
+	virtual void FineTickerCancel(TickReason reason);
 	virtual void SetMouseCapture(bool on);
 	virtual bool HaveMouseCapture();
 	virtual void SetTrackMouseLeaveEvent(bool on);
@@ -330,6 +343,7 @@ ScintillaWin::ScintillaWin(HWND hwnd) {
 	capturedMouse = false;
 	trackedMouseLeave = false;
 	TrackMouseEventFn = 0;
+	SetCoalescableTimerFn = 0;
 
 	linesPerScroll = 0;
 	wheelDelta = 0;   // Wheel delta from roll
@@ -385,8 +399,10 @@ void ScintillaWin::Initialise() {
 
 	// Find TrackMouseEvent which is available on Windows > 95
 	HMODULE user32 = ::GetModuleHandle(TEXT("user32.dll"));
-	if (user32)
+	if (user32) {
 		TrackMouseEventFn = (TrackMouseEventSig)::GetProcAddress(user32, "TrackMouseEvent");
+		SetCoalescableTimerFn = (SetCoalescableTimerSig)::GetProcAddress(user32, "SetCoalescableTimer");
+	}
 	if (TrackMouseEventFn == NULL) {
 		// Windows 95 has an emulation in comctl32.dll:_TrackMouseEvent
 		if (!commctrl32)
@@ -396,11 +412,16 @@ void ScintillaWin::Initialise() {
 				::GetProcAddress(commctrl32, "_TrackMouseEvent");
 		}
 	}
+	for (TickReason tr = tickCaret; tr <= tickDwell; tr = static_cast<TickReason>(tr + 1)) {
+		timers[tr] = 0;
+	}
 }
 
 void ScintillaWin::Finalise() {
 	ScintillaBase::Finalise();
-	SetTicking(false);
+	for (TickReason tr = tickCaret; tr <= tickDwell; tr = static_cast<TickReason>(tr + 1)) {
+		FineTickerCancel(tr);
+	}
 	SetIdle(false);
 #if defined(USE_D2D)
 	DropRenderTarget();
@@ -552,6 +573,31 @@ static int KeyTranslate(int keyIn) {
 	}
 }
 
+static bool BoundsContains(PRectangle rcBounds, HRGN hRgnBounds, PRectangle rcCheck) {
+	bool contains = true;
+	if (!rcCheck.Empty()) {
+		if (!rcBounds.Contains(rcCheck)) {
+			contains = false;
+		} else if (hRgnBounds) {
+			// In bounding rectangle so check more accurately using region
+			HRGN hRgnCheck = ::CreateRectRgn(static_cast<int>(rcCheck.left), static_cast<int>(rcCheck.top),
+				static_cast<int>(rcCheck.right), static_cast<int>(rcCheck.bottom));
+			if (hRgnCheck) {
+				HRGN hRgnDifference = ::CreateRectRgn(0, 0, 0, 0);
+				if (hRgnDifference) {
+					int combination = ::CombineRgn(hRgnDifference, hRgnCheck, hRgnBounds, RGN_DIFF);
+					if (combination != NULLREGION) {
+						contains = false;
+					}
+					::DeleteRgn(hRgnDifference);
+				}
+				::DeleteRgn(hRgnCheck);
+			}
+		}
+	}
+	return contains;
+}
+
 LRESULT ScintillaWin::WndPaint(uptr_t wParam) {
 	//ElapsedTime et;
 
@@ -575,7 +621,7 @@ LRESULT ScintillaWin::WndPaint(uptr_t wParam) {
 	}
 	rcPaint = PRectangle::FromInts(pps->rcPaint.left, pps->rcPaint.top, pps->rcPaint.right, pps->rcPaint.bottom);
 	PRectangle rcClient = GetClientRectangle();
-	paintingAllText = rcPaint.Contains(rcClient);
+	paintingAllText = BoundsContains(rcPaint, hRgnUpdate, rcClient);
 	if (technology == SC_TECHNOLOGY_DEFAULT) {
 		AutoSurface surfaceWindow(pps->hdc, this);
 		if (surfaceWindow) {
@@ -626,7 +672,6 @@ sptr_t ScintillaWin::HandleComposition(uptr_t wParam, sptr_t lParam) {
 	if (lParam & GCS_RESULTSTR) {
 		HIMC hIMC = ::ImmGetContext(MainHWND());
 		if (hIMC) {
-			const int maxLenInputIME = 200;
 			wchar_t wcs[maxLenInputIME];
 			LONG bytes = ::ImmGetCompositionStringW(hIMC,
 				GCS_RESULTSTR, wcs, (maxLenInputIME-1)*2);
@@ -659,6 +704,99 @@ sptr_t ScintillaWin::HandleComposition(uptr_t wParam, sptr_t lParam) {
 	return ::DefWindowProc(MainHWND(), WM_IME_COMPOSITION, wParam, lParam);
 }
 
+bool ScintillaWin::KoreanIME() {
+	const int codePage = InputCodePage();
+	return codePage == 949 || codePage == 1361;
+}
+
+sptr_t ScintillaWin::HandleCompositionKoreanIME(uptr_t, sptr_t lParam) {
+
+	// copy & paste by johnsonj
+	// Great thanks to
+	// jiniya from http://www.jiniya.net/tt/494  for DBCS input with AddCharUTF()
+	// BLUEnLIVE from http://zockr.tistory.com/1118 for UNDO and inOverstrike
+
+	HIMC hIMC = ::ImmGetContext(MainHWND());
+	if (!hIMC) {
+		return 0;
+	}
+
+	wchar_t wcs[maxLenInputIME];
+	int wides = 0;
+	bool compstrExist = false;
+
+	if (pdoc->TentativeActive()) {
+		pdoc->TentativeUndo();
+	} else {
+		// No tentative undo means start of this composition so
+		// fill in any virtual spaces.
+		bool tmpOverstrike = inOverstrike;
+		inOverstrike = false;         // not allow to be deleted twice.
+		AddCharUTF("", 0);
+		inOverstrike = tmpOverstrike;
+	}
+
+	view.imeCaretBlockOverride = false;
+	if (lParam & GCS_COMPSTR) {
+		long bytes = ::ImmGetCompositionStringW
+						   (hIMC, GCS_COMPSTR, wcs, maxLenInputIME);
+		wides = bytes / 2;
+		compstrExist = (wides != 0);
+	} else if (lParam & GCS_RESULTSTR) {
+		long bytes = ::ImmGetCompositionStringW
+						(hIMC, GCS_RESULTSTR, wcs, maxLenInputIME);
+		wides = bytes / 2;
+		compstrExist = (wides == 0);
+	}
+
+	if (wides >= 1) {
+
+		char hanval[maxLenInputIME];
+		unsigned int hanlen = 0;
+
+		if (IsUnicodeMode()) {
+			hanlen = UTF8Length(wcs, wides);
+			UTF8FromUTF16(wcs, wides, hanval, hanlen);
+			hanval[hanlen] = '\0';
+		} else {
+			hanlen = ::WideCharToMultiByte(InputCodePage(), 0,
+						wcs, wides, hanval, sizeof(hanval) - 1, 0, 0);
+			hanval[hanlen] = '\0';
+		}
+
+		if (compstrExist) {
+			view.imeCaretBlockOverride = true;
+
+			bool tmpRecordingMacro = recordingMacro;
+			recordingMacro = false;
+			pdoc->TentativeStart();
+			AddCharUTF(hanval, hanlen);
+			recordingMacro = tmpRecordingMacro;
+
+			for (size_t r = 0; r < sel.Count(); r++) { // for block caret
+				int positionInsert = sel.Range(r).Start().Position();
+				sel.Range(r).caret.SetPosition(positionInsert - hanlen);
+				sel.Range(r).anchor.SetPosition(positionInsert - hanlen);
+			}
+		} else {
+			AddCharUTF(hanval, hanlen);
+		}
+	}
+
+	// set the candidate window position for HANJA while composing.
+	Point pos = PointMainCaret();
+	CANDIDATEFORM CandForm;
+	CandForm.dwIndex = 0;
+	CandForm.dwStyle = CFS_CANDIDATEPOS;
+	CandForm.ptCurrentPos.x = static_cast<int>(pos.x);
+	CandForm.ptCurrentPos.y = static_cast<int>(pos.y + vs.lineHeight);
+	::ImmSetCandidateWindow(hIMC, &CandForm);
+
+	ShowCaretAtCurrentPosition();
+	::ImmReleaseContext(MainHWND(), hIMC);
+	return 0;
+}
+
 // Translate message IDs from WM_* and EM_* to SCI_* so can partly emulate Windows Edit control
 static unsigned int SciMessageFromEM(unsigned int iMessage) {
 	switch (iMessage) {
@@ -883,12 +1021,10 @@ sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 			return 0;
 
 		case WM_TIMER:
-			if (wParam == standardTimerID && timer.ticking) {
-				Tick();
-			} else if (wParam == idleTimerID && idler.state) {
+			if (wParam == idleTimerID && idler.state) {
 				SendMessage(MainHWND(), SC_WIN_IDLE, 0, 1);
 			} else {
-				return 1;
+				TickFor(static_cast<TickReason>(wParam - fineTimerStart));
 			}
 			break;
 
@@ -1077,6 +1213,12 @@ sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 					SetFocusState(false);
 					DestroySystemCaret();
 				}
+				// Explicitly complete any IME composition
+				HIMC hIMC = ImmGetContext(MainHWND());
+				if (hIMC) {
+					::ImmNotifyIME(hIMC, NI_COMPOSITIONSTR, CPS_COMPLETE, 0);
+					::ImmReleaseContext(MainHWND(), hIMC);
+				}
 			}
 			break;
 
@@ -1092,6 +1234,9 @@ sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 			break;
 
 		case WM_IME_STARTCOMPOSITION: 	// dbcs
+			if (KoreanIME()) {
+				return 0;
+			}
 			ImeStartComposition();
 			return ::DefWindowProc(MainHWND(), iMessage, wParam, lParam);
 
@@ -1100,7 +1245,11 @@ sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 			return ::DefWindowProc(MainHWND(), iMessage, wParam, lParam);
 
 		case WM_IME_COMPOSITION:
-			return HandleComposition(wParam, lParam);
+			if (KoreanIME()) {
+				return HandleCompositionKoreanIME(wParam, lParam);
+			} else {
+				return HandleComposition(wParam, lParam);
+			}
 
 		case WM_IME_CHAR: {
 				AddCharBytes(HIBYTE(wParam), LOBYTE(wParam));
@@ -1136,6 +1285,16 @@ sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 			capturedMouse = false;
 			return 0;
 
+		case WM_IME_SETCONTEXT:
+			if (KoreanIME()) {
+				if (wParam) {
+					LPARAM NoImeWin = lParam;
+					NoImeWin = NoImeWin & (~ISC_SHOWUICOMPOSITIONWINDOW);
+					return ::DefWindowProc(MainHWND(), iMessage, wParam, NoImeWin);
+				}
+			}
+			return ::DefWindowProc(MainHWND(), iMessage, wParam, lParam);
+
 		// These are not handled in Scintilla and its faster to dispatch them here.
 		// Also moves time out to here so profile doesn't count lots of empty message calls.
 
@@ -1146,7 +1305,6 @@ sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 		case WM_NCPAINT:
 		case WM_NCMOUSEMOVE:
 		case WM_NCLBUTTONDOWN:
-		case WM_IME_SETCONTEXT:
 		case WM_IME_NOTIFY:
 		case WM_SYSCOMMAND:
 		case WM_WINDOWPOSCHANGING:
@@ -1284,20 +1442,34 @@ sptr_t ScintillaWin::DefWndProc(unsigned int iMessage, uptr_t wParam, sptr_t lPa
 	return ::DefWindowProc(MainHWND(), iMessage, wParam, lParam);
 }
 
-void ScintillaWin::SetTicking(bool on) {
-	if (timer.ticking != on) {
-		timer.ticking = on;
-		if (timer.ticking) {
-			timer.tickerID = ::SetTimer(MainHWND(), standardTimerID, timer.tickSize, NULL)
-				? reinterpret_cast<TickerID>(standardTimerID) : 0;
-		} else {
-			::KillTimer(MainHWND(), reinterpret_cast<uptr_t>(timer.tickerID));
-			timer.tickerID = 0;
-		}
-	}
-	timer.ticksToWait = caret.period;
+/**
+* Report that this Editor subclass has a working implementation of FineTickerStart.
+*/
+bool ScintillaWin::FineTickerAvailable() {
+	return true;
 }
 
+bool ScintillaWin::FineTickerRunning(TickReason reason) {
+	return timers[reason] != 0;
+}
+
+void ScintillaWin::FineTickerStart(TickReason reason, int millis, int tolerance) {
+	FineTickerCancel(reason);
+	if (SetCoalescableTimerFn && tolerance) {
+		timers[reason] = SetCoalescableTimerFn(MainHWND(), fineTimerStart + reason, millis, NULL, tolerance);
+	} else {
+		timers[reason] = ::SetTimer(MainHWND(), fineTimerStart + reason, millis, NULL);
+	}
+}
+
+void ScintillaWin::FineTickerCancel(TickReason reason) {
+	if (timers[reason]) {
+		::KillTimer(MainHWND(), timers[reason]);
+		timers[reason] = 0;
+	}
+}
+
+
 bool ScintillaWin::SetIdle(bool on) {
 	// On Win32 the Idler is implemented as a Timer on the Scintilla window.  This
 	// takes advantage of the fact that WM_TIMER messages are very low priority,
@@ -1338,33 +1510,17 @@ void ScintillaWin::SetTrackMouseLeaveEvent(bool on) {
 		tme.cbSize = sizeof(tme);
 		tme.dwFlags = TME_LEAVE;
 		tme.hwndTrack = MainHWND();
+		tme.dwHoverTime = HOVER_DEFAULT;	// Unused but triggers Dr. Memory if not initialized
 		TrackMouseEventFn(&tme);
 	}
 	trackedMouseLeave = on;
 }
 
 bool ScintillaWin::PaintContains(PRectangle rc) {
-	bool contains = true;
-	if ((paintState == painting) && (!rc.Empty())) {
-		if (!rcPaint.Contains(rc)) {
-			contains = false;
-		} else {
-			// In bounding rectangle so check more accurately using region
-			HRGN hRgnRange = ::CreateRectRgn(static_cast<int>(rc.left), static_cast<int>(rc.top), static_cast<int>(rc.right), static_cast<int>(rc.bottom));
-			if (hRgnRange) {
-				HRGN hRgnDest = ::CreateRectRgn(0, 0, 0, 0);
-				if (hRgnDest) {
-					int combination = ::CombineRgn(hRgnDest, hRgnRange, hRgnUpdate, RGN_DIFF);
-					if (combination != NULLREGION) {
-						contains = false;
-					}
-					::DeleteRgn(hRgnDest);
-				}
-				::DeleteRgn(hRgnRange);
-			}
-		}
+	if (paintState == painting) {
+		return BoundsContains(rcPaint, hRgnUpdate, rc);
 	}
-	return contains;
+	return true;
 }
 
 void ScintillaWin::ScrollText(int /* linesToMove */) {
@@ -2907,25 +3063,31 @@ int Scintilla_RegisterClasses(void *hInstance) {
 	return result;
 }
 
-// This function is externally visible so it can be called from container when building statically.
-int Scintilla_ReleaseResources() {
+static int ResourcesRelease(bool fromDllMain) {
 	bool result = ScintillaWin::Unregister();
 	if (commctrl32) {
 		FreeLibrary(commctrl32);
 		commctrl32 = NULL;
 	}
-	Platform_Finalise();
+	Platform_Finalise(fromDllMain);
 	return result;
 }
 
+// This function is externally visible so it can be called from container when building statically.
+int Scintilla_ReleaseResources() {
+	return ResourcesRelease(false);
+}
+
 #ifndef STATIC_BUILD
-extern "C" int APIENTRY DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID) {
+extern "C" int APIENTRY DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpvReserved) {
 	//Platform::DebugPrintf("Scintilla::DllMain %d %d\n", hInstance, dwReason);
 	if (dwReason == DLL_PROCESS_ATTACH) {
 		if (!Scintilla_RegisterClasses(hInstance))
 			return FALSE;
 	} else if (dwReason == DLL_PROCESS_DETACH) {
-		Scintilla_ReleaseResources();
+		if (lpvReserved == NULL) {
+			ResourcesRelease(true);
+		}
 	}
 	return TRUE;
 }
